/*
*****************************************************************************************************************************************************
Projet Centrale domotique DomoMaison 2017 pour carte Arduino Mega 2560 et carte Ethernet (W5100)
Version 1.70 du 06/09/2017
Copyright 2017 PRIN Jean-Luc
Ce programme fonctionne avec le protocole Websockets (HTML5).
*****************************************************************************************************************************************************
*/
/*
*****************************************************************************************************************************************************
Modules contenant les fonctions nécessaires à la gestion de l'électricité, de la téléInformation, de l'enregistrement des données des dernières 48
heures, des données journalières sur carte SD (consommation électrique, températures intérieures et extérieures, compteur d'eau
*****************************************************************************************************************************************************
*/
// **************************************************************************************************************************************************
// ------------------------------ DEBUT DES FONCTIONS ELECTRICITE INCLUES DANS LA BOUCLE LOOP -------------------------------------------------------
// **************************************************************************************************************************************************
// -------------- Début de la fonction mise_jour_teleinfo_h48 (mise à jour téléInfo et données sur les 48 heures) -----------------------------------
void mise_jour_teleinfo_h48()
{
	//----Traitement index de consommation horaire pour les compteurs consommation sur les 48 heures précédentes consécutives pour graphe
    Epoch_Teleinfo = Epoch_Local;
    Heure_Epoch_Teleinfo = hour(Epoch_Teleinfo);
        #ifdef DEBUG_ELEC
            Serial.println(F("Mise jour teleinfo"));
        #endif
    //décalage des deltas HC,HP et heure.
    for (int j=0; j<47; j++){
        Delta_IndexHC1[47 - j] = Delta_IndexHC1[46 - j];
        Delta_IndexHP1[47 - j] = Delta_IndexHP1[46 - j];
        IntHeure[47 - j] = IntHeure[46 - j];
        Temp_Ext[47 - j] = Temp_Ext[46 - j];
        Temp_Int[47 - j] = Temp_Int[46 - j];
    }
    IntHeure[0] = Hours;
    //re-initialisation des index pour l'heure passée.
    IndexHC1 = Ejphn1;      // pour BASE remplacer ejphn1 par hchc1
    IndexHP1 = Ejphp1;      // pour BASE remplacer ejphp1 par hchp1
}
// -------------- Fin de la fonction mise_jour_teleinfo_h48 (mise à jour téléInfo et données sur les 48 heures) -------------------------------------

// Début de la fonction calcul_diff_affichage_graphe (calcul des différences de façon permanente pour affichage des informations de l'heure courante -
void calcul_diff_affichage_graphe()
{
    if (IndexHC1 != 0){
        Tempo_DeltaHC = Delta_IndexHC1[0];
        Delta_IndexHC1[0] = Ejphn1 - IndexHC1;
			if (Delta_IndexHC1[0] < 0) Delta_IndexHC1[0] = 0;
            if (Delta_IndexHC1[0] > 15000) Delta_IndexHC1[0] = Tempo_DeltaHC;
    }
    else Delta_IndexHC1[0] = 0;
    if (IndexHP1 != 0){
        Delta_IndexHP1[0] = Ejphp1 - IndexHP1;
			if (Delta_IndexHP1[0] < 0) Delta_IndexHP1[0] = 0;
    }
    else Delta_IndexHP1[0] = 0;
    Temp_Ext[0] = int(TempT[NumS_Ext] * 10);
    Temp_Int[0] = int(TempT[NumS_Int] * 10);
}
// Fin de la fonction calcul_diff_affichage_graphe (calcul des différences de façon permanente pour affichage des informations de l'heure courante --

// --------------------------- Début de la fonction enreg_infoJour_SD (enregistrement des données journalières sur carte SD) ------------------------
void enreg_infoJour_SD()
{
		//test si premier passage pour ne pas enregistrer des données cumulées avec la veille à partir de la date-heure courante.
		if(Jour_Epoch_Teleinfo != 0){
			//calcul du jour correspondant à la veille.
			Epoch_Veille = Epoch_Local - 86400;                     //soit 24h * 3600 secondes.
			LogFile = SD.open("/");
			LogFile.rewindDirectory();
			memcpy(&Nom_Fichier_Rep[0], "sj", 2);
			snprintf(Annee_Str,5,"%d",year(Epoch_Veille));
			memcpy(&Nom_Fichier_Rep[2], &Annee_Str,4);
			Nom_Fichier_Rep[6] = '\0';
			//Création du répertoire
			if (!SD.exists(Nom_Fichier_Rep)) {
				if (!SD.mkdir(Nom_Fichier_Rep)) {
					//Serial.println("Echec creation");//Pour test.
				}
				//else Serial.println("Creation realisee.");//Pour test.
			}
			//Test si le fichier du mois  du jour précédent existe sinon le créer.
			memcpy(&Nom_Fichier_Rep[0], "sj", 2);
			snprintf(Annee_Str,5,"%d",year(Epoch_Veille));
			memcpy(&Nom_Fichier_Rep[2], &Annee_Str,4);
			memcpy(&Nom_Fichier_Rep[6], "/", 1);
			memcpy(&Nom_Fichier_Rep[7], "sj", 2);
			snprintf(Mois_Str,3,"%02d",month(Epoch_Veille));
			memcpy(&Nom_Fichier_Rep[9], &Mois_Str,2);
			memcpy(&Nom_Fichier_Rep[11], ".dat", 4);
			Nom_Fichier_Rep[15] = '\0';
			//création du fichier si il n'existe pas.
			if (!SD.exists(Nom_Fichier_Rep)) {
				LogFile = SD.open(Nom_Fichier_Rep, FILE_WRITE);
				LogFile.close();
				//Controle de la creation du fichier.
				if (SD.exists(Nom_Fichier_Rep)){
				}
			}

			//ajout enregistrement au format CSV.
			//horodate.
			memcpy(&Enr_Log[0], &Annee_Str,4);
			memcpy(&Enr_Log[4], "-", 1);
			memcpy(&Enr_Log[5], &Mois_Str,2);
			memcpy(&Enr_Log[7], "-", 1);
			snprintf(&Enr_Log[8],3,"%02d",day(Epoch_Veille));
			memcpy(&Enr_Log[10], ";", 1);
			//enregistrement consommation Heures creuses du jour passé.
			snprintf(&Enr_Log[11],10,"%9ld",Ejphn1 - Jour_IndexHC1);                 // pour BASE remplacer ejphn1 par hchc1
			//séparateur csv.
			memcpy(&Enr_Log[20], ";", 1);
			//enregistrement consommation Heures pleines du jour passé.
			snprintf(&Enr_Log[21],10,"%9ld",Ejphp1 - Jour_IndexHP1);                 // pour BASE remplacer ejphp1 par hchp1
			//séparateur csv.
			memcpy(&Enr_Log[30], ";", 1);
			//enregistrement index Heures creuses du jour passé.
			snprintf(&Enr_Log[31],10,"%9ld",Ejphn1);                                 // pour BASE remplacer ejphn1 par hchc1
			//séparateur csv.
			memcpy(&Enr_Log[40], ";", 1);
			//enregistrement index Heures pleines du jour passé.
			snprintf(&Enr_Log[41],10,"%9ld",Ejphp1);                                   // pour BASE remplacer ejphp1 par hchp1
			//séparateur csv.
			memcpy(&Enr_Log[50], ";", 1);
			//enregistrement temperature extérieure minimale du jour passé.
			snprintf(&Enr_Log[51],5,"%4d",Temp_Ext_Mini);//
			memcpy(&Enr_Log[55], ";", 1);
			//enregistrement temperature extérieure maximale du jour passé.
			snprintf(&Enr_Log[56],5,"%4d",Temp_Ext_Maxi);//
			memcpy(&Enr_Log[60], ";", 1);
			//enregistrement temperature intérieure minimale du jour passé.
			snprintf(&Enr_Log[61],5,"%4d",Temp_Int_Mini);//
			memcpy(&Enr_Log[65], ";", 1);
			//enregistrement temperature intérieure maximale du jour passé.
			snprintf(&Enr_Log[66],5,"%4d",Temp_Int_Maxi);//
			memcpy(&Enr_Log[70], ";", 1);
			//enregistrement du compteur d'eau.
			snprintf(&Enr_Log[71],10,"%9lu",Compteur_Pulse_1);      //compteur d'eau.
			Enr_Log[80] = '\0';
			#ifdef DEBUG_ELEC
				Serial.print(F("enregistrement dans la SDcard="));
				Serial.println(Enr_Log);
			#endif

			//Enregistrement dans la SDcard
			LogFile = SD.open(Nom_Fichier_Rep, FILE_WRITE);
			 // if the file is available, write to it:
			if (LogFile) {
				LogFile.println(Enr_Log);
				LogFile.close();
			}
		}
		// Compteurs consommation et production --- re-initialisation des index pour le jour courant ou la référence au démarrage.
		Jour_IndexHC1 = Ejphn1;                             // pour BASE remplacer ejphn1 par hchc1
		Jour_IndexHP1 = Ejphp1;                             // pour BASE remplacer ejphp1 par hchp1

		// ré-initialisation des températures mini et maxi pour la journée suivante.
        Temp_Ext_Mini=1000;
        Temp_Ext_Maxi=-500;
        Temp_Int_Mini=1000;
        Temp_Int_Maxi=-500;

		//mise à jour de la date epoch_teleinfo courante.
		Jour_Epoch_Teleinfo = Epoch_Local;
		Day_Jour_Teleinfo = day(Jour_Epoch_Teleinfo);

}
// --------------------------- Fin de la fonction enreg_infoJour_SD (enregistrement des données journalières sur carte SD) --------------------------
// **************************************************************************************************************************************************
// ------------------------------ FIN DES FONCTIONS ELECTRICITE INCLUES DANS LA BOUCLE LOOP ---------------------------------------------------------
// **************************************************************************************************************************************************


// **************************************************************************************************************************************************
// ------------------------------ DEBUT DES FONCTIONS TELEINFORMATION -------------------------------------------------------------------------------
// **************************************************************************************************************************************************
// ------------------ Début de la fonction read_teleinfo_consommation (récupération des données de groupe téléinfo EDF consommation) ----------------
void read_teleinfo_consommation()
{
	if (Serial1.available() > 0){
		InByte1 = Serial1.read();
		if(InByte1 == Deb) Bufflen1 = 0;
		Buffin1[Bufflen1] = InByte1;
		Bufflen1++;
		if (Bufflen1 > 22) Bufflen1 = 0;                                //traitement anomalie sur la longueur d'un groupe du téléinformation.
		else if(InByte1 == Fin && Bufflen1 > 5){                        // Fin de ligne trouvée.
			if (cksum(Buffin1,Bufflen1 - 1) == Buffin1[Bufflen1 - 2]){  // Test du CKsum
				traitbuf1(Buffin1);                                     // CkSum OK => Visualisation de la Trame
			}
		}
	}
}
// ------------------ Début de la fonction read_teleinfo_consommation (récupération des données de groupe téléinfo EDF consommation) ----------------

// ------------------------- Début de la fonction cksum (calcul Checksum Téléinfo) ------------------------------------------------------------------
char cksum(char *Buff, int Len){
	int i;
	char Sum = 0;

        for (i=1; i<(Len - 2); i++) Sum = Sum + Buff[i];
    Sum = (Sum & 0x3F) + 0x20;
    return(Sum);
}
// ------------------------- Fin de la fonction cksum (calcul Checksum Téléinfo) --------------------------------------------------------------------

// ------------------------- Début de la fonction traitbuf1 (traitement des données recues par la téléInformation -----------------------------------
void traitbuf1(char *Buff)
{
    if (strncmp("ADCO ", &Buff[1] , 5) == 0){
		strncpy(Adco1, &Buff[6], 12);
		Adco1[12] = '\0';
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("ADCO = "));
                Serial.println(Adco1);
            #endif
    }
    else if (strncmp("OPTARIF ", &Buff[1] , 8) == 0){
	    strncpy(Optarif1, &Buff[9], 4);
	    Optarif1[4] = '\0';
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("OPTARIF ="));
                Serial.println(Optarif1);
            #endif
    }
    else if (strncmp("ISOUSC ", &Buff[1] , 7) == 0){
	    Isousc1 = atol(&Buff[8]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("ISOUSC = "));
                Serial.println(Isousc1,DEC);
            #endif
    }
    else if (strncmp("HCHC ", &Buff[1] , 5) == 0){
		Hchc1 = atol(&Buff[6]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("HCHC = "));
                Serial.println(Hchc1,DEC);
            #endif
    }
    else if (strncmp("HCHP ", &Buff[1] , 5) == 0){
	    Hchp1 = atol(&Buff[6]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("HCHP = "));
                Serial.println(Hchp1,DEC);
            #endif
    }
    else if (strncmp("EJPHN ", &Buff[1] , 6) == 0){
		Ejphn1 = atol(&Buff[7]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("EJPHN = "));
                Serial.println(Ejphn1,DEC);
            #endif
    }
    else if (strncmp("EJPHPM ", &Buff[1] , 7) == 0){
	    Ejphp1 = atol(&Buff[8]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("EJPHPM = "));
                Serial.println(Ejphp1,DEC);
            #endif
    }
    else if (strncmp("PTEC ", &Buff[1] , 5) == 0){
	    strncpy(Ptec1, &Buff[6], 4);
	    Ptec1[4] = '\0';
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("PTEC = "));
                Serial.println(Ptec1);
            #endif
    }
    else if (strncmp("IINST ", &Buff[1] , 6) == 0){
	    Iinst1 = atol(&Buff[7]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("IINST = "));
                Serial.println(Iinst1,DEC);
            #endif
    }
    else if (strncmp("IMAX ", &Buff[1] , 5 )== 0){
	    Imax1 = atol(&Buff[6]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("IMAX = "));
                Serial.println(Imax1,DEC);
            #endif
    }
    else if (strncmp("PAPP ", &Buff[1] , 5) == 0){
	    Papp1 = atol(&Buff[6]);
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("PAPP = "));
                Serial.println(Papp1,DEC);
            #endif
    }
    else if (strncmp("HHPHC ", &Buff[1] , 6) == 0){
	    Hhphc1 = Buff[7];
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("HHPHC = "));
                Serial.println(Hhphc1);
            #endif
    }
    else if (strncmp("MOTDETAT ", &Buff[1] , 9) == 0){
	    strncpy(Motdetat1, &Buff[10], 6);
	    Motdetat1[6] = '\0';
		Compteur_Detection_Teleinfo1 = 0;
		Indic_Teleinfo1 = 1;
            #ifdef DEBUG_ELEC
                Serial.print(F("MOTDETAT = "));
                Serial.println(Motdetat1);
            #endif
    }
}
// ------------------------- Fin de la fonction traitbuf1 (traitement des données recues par la téléInformation -----------------------------------
// **************************************************************************************************************************************************
// ------------------------------ FIN DES FONCTIONS TELEINFORMATION ---------------------------------------------------------------------------------
// **************************************************************************************************************************************************

// **************************************************************************************************************************************************
// ------------------------------ DEBUT DES FONCTIONS D'ECHANGE AVEC LE SERVEUR ---------------------------------------------------------------------
// **************************************************************************************************************************************************
// ------------------ Début de la fonction envoi_info_elect (envoi de la conso électrique instantanée et du signal EJP) -----------------------------
void envoi_info_elect()
{
    // Informations pour le barreGraphe d'instensité instannée
    //IINST
    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INFEL",5);
	EvtServer[11] = Separator;
    strncpy(&EvtServer[12], "01", 2);
    EvtServer[14] = Separator;
        if (Indic_Teleinfo1 == 1){
            snprintf(&EvtServer[15],4,"%3ld",Iinst1);
        }
        else{
            strncpy(&EvtServer[15], "***", 3);
        }
    EvtServer[18] = Separator;
    //itoa(Ejp,Buff,10);
    //strncpy(&EvtServer[19], Buff,1);                  // signal ejp provenant de la borne PIN_EJP
    strncpy(&EvtServer[19], Ptec1, 4);                  // signa EJP provenant de la téléInfo (application consommation en cours)
    //EvtServer[20] = ETX;
    //EvtServer[21] = '\0';
    EvtServer[23] = ETX;
    EvtServer[24] = '\0';
    //Vérification de la connexion du client.
        if (wsServer.isConnected()) {
            //wsServer.send(EvtServer,21);      //Envoi
            wsServer.send(EvtServer,24);        //Envoi
        }
        #ifdef DEBUG_ELEC
            Serial.print(F("chaine IINST et EJP = "));
            Serial.println(EvtServer);
        #endif

	// Informations pour la tendance de consommation électrique
	// Abonnement EDF de base (Heures creuses et Heures pleines)
		if (ABONNEMENT == "BASE") {
			//HCHC
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INFEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "02", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "HCHC",4);
			EvtServer[19] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[20],10,"%9ld",Hchc1);
					EvtServer[29] = ETX;
					EvtServer[30] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,30);    //Envoi
						}
				}
				else{
					strncpy(&EvtServer[20], "***", 3);
					EvtServer[23] = ETX;
					EvtServer[24] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,24);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine HCHC = "));
					Serial.println(EvtServer);
				#endif
			//
			//HCHP
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INFEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "02", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "HCHP",4);
			EvtServer[19] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[20],10,"% 9ld",Hchp1);
					EvtServer[29] = ETX;
					EvtServer[30] = '\0';
					//Vérification de la connexion du client.
					if (wsServer.isConnected()) {
						wsServer.send(EvtServer,30);    //Envoi
					}
				}
				else{
					strncpy(&EvtServer[20], "***", 3);
					EvtServer[23] = ETX;
					EvtServer[24] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,24);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine HCHP = "));
					Serial.println(EvtServer);
				#endif
		}
                //
	// Abonnement EDF EJP (Heures pleines sur 22 jours, reste en Heures creuses)
		if (ABONNEMENT == "EJP") {
			//EJPHN
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INFEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "02", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "EJPHN",5);
			EvtServer[20] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[21],10,"%9ld",Ejphn1);
					EvtServer[30] = ETX;
					EvtServer[31] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,31);    //Envoi
						}
				}
				else{
					strncpy(&EvtServer[21], "***", 3);
					EvtServer[24] = ETX;
					EvtServer[25] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,25);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine EJPHN = "));
					Serial.println(EvtServer);
				#endif
			//
			//EJPHP
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INFEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "02", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "EJPHP",5);
			EvtServer[20] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[21],10,"% 9ld",Ejphp1);
					EvtServer[30] = ETX;
					EvtServer[31] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,31);    //Envoi
                      }
				}
				else{
					strncpy(&EvtServer[21], "***", 3);
					EvtServer[24] = ETX;
					EvtServer[25] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,25);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine EJPHP = "));
					Serial.println(EvtServer);
				#endif
		}

}
// ------------------ Fin de la fonction envoi_info_elect (envoi de la conso électrique instantanée et du signal EJP) -------------------------------

// ------------------- Début de la fonction envoi_web_teleinfo (transmission de toutes les données Téléinfo à l'interface Web -----------------------
void envoi_web_teleinfo()
{
	//ADCO
	EvtServer[0] = STX;
	strncpy(&EvtServer[1], NumCard, 4);
	EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INTEL",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "01", 2);
	EvtServer[14] = Separator;
	strncpy(&EvtServer[15], "ADCO",4);
	EvtServer[19] = Separator;
		if (Indic_Teleinfo1 == 1){
			strncpy(&EvtServer[20], Adco1, 12);
			EvtServer[32] = ETX;
			EvtServer[33] = '\0';
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,33);    //Envoi
				}
		}
		else{
			strncpy(&EvtServer[20], "***", 3);
			EvtServer[23] = ETX;
			EvtServer[24] = '\0';
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,24);    //Envoi
				}
		}
		#ifdef DEBUG_ELEC
			Serial.print(F("chaine ADCO = "));
			Serial.println(EvtServer);
		#endif
	//
	//OPTARIF
	EvtServer[0] = STX;
	strncpy(&EvtServer[1], NumCard, 4);
	EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INTEL",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "01", 2);
	EvtServer[14] = Separator;
	strncpy(&EvtServer[15], "OPTARIF",7);
	EvtServer[22] = Separator;
		if (Indic_Teleinfo1 == 1){
			strncpy(&EvtServer[23],Optarif1,4);
			EvtServer[27] = ETX;
			EvtServer[28] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,28);    //Envoi
				}
		}
		else{
			strncpy(&EvtServer[23], "***", 3);
			EvtServer[26] = ETX;
			EvtServer[27] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,27);    //Envoi
				}
		}
		#ifdef DEBUG_ELEC
			Serial.print(F("chaine OPTARIF = "));
			Serial.println(EvtServer);
		#endif
	//
	//ISOUSC
	EvtServer[0] = STX;
	strncpy(&EvtServer[1], NumCard, 4);
	EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INTEL",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "01", 2);
	EvtServer[14] = Separator;
	strncpy(&EvtServer[15], "ISOUSC",6);
	EvtServer[21] = Separator;
		if (Indic_Teleinfo1 == 1){
			snprintf(&EvtServer[22],10,"%2ld",Isousc1);
			EvtServer[24] = ETX;
			EvtServer[25] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,25);    //Envoi
				}
		}
		else{
			strncpy(&EvtServer[22], "***", 3);
			EvtServer[25] = ETX;
			EvtServer[26] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,26);    //Envoi
				}
		}
		#ifdef DEBUG_ELEC
			Serial.print(F("chaine ISOUSC = "));
			Serial.println(EvtServer);
		#endif
	//
	// Abonnement EDF de base (Heures creuses et Heures pleines)
		if (ABONNEMENT == "BASE") {
			//HCHC
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INTEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "01", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "HCHC",4);
			EvtServer[19] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[20],10,"%9ld",Hchc1);
					EvtServer[29] = ETX;
					EvtServer[30] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,30);    //Envoi
						}
				}
				else{
					strncpy(&EvtServer[20], "***", 3);
					EvtServer[23] = ETX;
					EvtServer[24] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,24);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine HCHC = "));
					Serial.println(EvtServer);
				#endif
			//
			//HCHP
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INTEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "01", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "HCHP",4);
			EvtServer[19] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[20],10,"% 9ld",Hchp1);
					EvtServer[29] = ETX;
					EvtServer[30] = '\0';
					//Vérification de la connexion du client.
					if (wsServer.isConnected()) {
						wsServer.send(EvtServer,30);    //Envoi
					}
				}
				else{
					strncpy(&EvtServer[20], "***", 3);
					EvtServer[23] = ETX;
					EvtServer[24] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,24);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine HCHP = "));
					Serial.println(EvtServer);
				#endif
		}
                //
	// Abonnement EDF EJP (Heures pleines sur 22 jours, reste en Heures creuses)
		if (ABONNEMENT == "EJP") {
			//EJPHN
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INTEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "01", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "EJPHN",5);
			EvtServer[20] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[21],10,"%9ld",Ejphn1);
					EvtServer[30] = ETX;
					EvtServer[31] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,31);    //Envoi
						}
				}
				else{
					strncpy(&EvtServer[21], "***", 3);
					EvtServer[24] = ETX;
					EvtServer[25] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,25);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine EJPHN = "));
					Serial.println(EvtServer);
				#endif
			//
			//EJPHP
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INTEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "01", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "EJPHP",5);
			EvtServer[20] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[21],10,"% 9ld",Ejphp1);
					EvtServer[30] = ETX;
					EvtServer[31] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,31);    //Envoi
                      }
				}
				else{
					strncpy(&EvtServer[21], "***", 3);
					EvtServer[24] = ETX;
					EvtServer[25] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,25);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine EJPHP = "));
					Serial.println(EvtServer);
				#endif
		}
	//
	//IINST
	EvtServer[0] = STX;
	strncpy(&EvtServer[1], NumCard, 4);
	EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INTEL",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "01", 2);
	EvtServer[14] = Separator;
	strncpy(&EvtServer[15], "IINST",5);//
	EvtServer[20] = Separator;
		if (Indic_Teleinfo1==1){
			snprintf(&EvtServer[21],4,"%3ld",Iinst1);
			EvtServer[24] = ETX;
			EvtServer[25] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,25);    //Envoi
				}
		}
		else{
			strncpy(&EvtServer[21], "***", 3);
			EvtServer[24] = ETX;
			EvtServer[25] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,24);    //Envoi
				}
		}
		#ifdef DEBUG_ELEC
			Serial.print(F("chaine IINST = "));
			Serial.println(EvtServer);
		#endif
	//
	//IMAX
	EvtServer[0] = STX;
	strncpy(&EvtServer[1], NumCard, 4);
	EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INTEL",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "01", 2);
	EvtServer[14] = Separator;
	strncpy(&EvtServer[15], "IMAX",4);
	EvtServer[19] = Separator;
		if (Indic_Teleinfo1 == 1){
			snprintf(&EvtServer[20],4,"%3ld",Imax1);
			EvtServer[23] = ETX;
			EvtServer[24] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,24);    //Envoi
				}
		}
		else{
			strncpy(&EvtServer[20], "***", 3);
			EvtServer[23] = ETX;
			EvtServer[24] = '\0';
			//Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,24);    //Envoi
				}
		}
		#ifdef DEBUG_ELEC
			Serial.print(F("chaine IMAX = "));
			Serial.println(EvtServer);
		#endif
	//
	//PAPP
		if (TYPE_COMPTEUR == "NOUV") {
			EvtServer[0] = STX;
			strncpy(&EvtServer[1], NumCard, 4);
			EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "INTEL",5);
            EvtServer[11] = Separator;
			strncpy(&EvtServer[12], "01", 2);
			EvtServer[14] = Separator;
			strncpy(&EvtServer[15], "PAPP",4);
			EvtServer[19] = Separator;
				if (Indic_Teleinfo1 == 1){
					snprintf(&EvtServer[20],6,"%5ld",Papp1);
					EvtServer[25] = ETX;
					EvtServer[26] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,26);    //Envoi
						}
				}
				else{
					strncpy(&EvtServer[20], "***", 3);
					EvtServer[23] = ETX;
					EvtServer[24] = '\0';
					//Vérification de la connexion du client.
						if (wsServer.isConnected()) {
							wsServer.send(EvtServer,24);    //Envoi
						}
				}
				#ifdef DEBUG_ELEC
					Serial.print(F("chaine PAPP = "));
					Serial.println(EvtServer);
				#endif
		}
	//
	//PTEC
	EvtServer[0] = STX;
	strncpy(&EvtServer[1], NumCard, 4);
	EvtServer[5] = Separator;
	strncpy(&EvtServer[6], "INTEL",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "01", 2);
	EvtServer[14] = Separator;
	strncpy(&EvtServer[15], "PTEC",4);
	EvtServer[19] = Separator;
		if (Indic_Teleinfo1 == 1){
			strncpy(&EvtServer[20], Ptec1, 4);
			EvtServer[24] = ETX;
			EvtServer[25] = '\0';
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,25);    //Envoi
				}
		}
		else{
			strncpy(&EvtServer[20], "***", 3);
			EvtServer[23] = ETX;
			EvtServer[24] = '\0';
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,24);    //Envoi
				}
		}
		#ifdef DEBUG_ELEC
			Serial.print(F("chaine PTEC = "));
			Serial.println(EvtServer);
		#endif
}
// ------------------- Fin de la fonction envoi_web_teleinfo (transmission de toutes les données Téléinfo à l'interface Web -------------------------

// ------------------- Début de la fonction envoiDonnees48 (envoi des données enregistrées lors des dernières 48 heures) ----------------------------
void envoiDonnees48()
{
    byte j;

        for (j=0; j<48; j++){
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "EDO48",5);                       //code informations horaires.
            EvtServer[11] = Separator;
            snprintf(&EvtServer[12],3,"%02d",j);                    //rang de l'information.
            EvtServer[14] = Separator;
            snprintf(&EvtServer[15],3,"%02d",IntHeure[j]);          // heure d'enregistrement des données
            EvtServer[17] = Separator;
            snprintf(&EvtServer[18],10,"%9ld",Delta_IndexHC1[j]);   //consommation horaire Heures creuses.
            EvtServer[27] = Separator;
            snprintf(&EvtServer[28],10,"%9ld",Delta_IndexHP1[j]);   //consommation horaire Heures pleines.
            EvtServer[37] = Separator;
            snprintf(&EvtServer[38],5,"%4d",Temp_Ext[j]);           //température extérieure.
            EvtServer[42] = Separator;
            snprintf(&EvtServer[43],5,"%4d",Temp_Int[j]);           //température intérieure.
            EvtServer[47] = ETX;
            EvtServer[48] = '\0';
            //Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,48);                    //Envoi
				}
				#ifdef DEBUG_ELEC
                    Serial.println(EvtServer);
				#endif
        }
}
// ------------------- Fin de la fonction envoiDonnees48 (envoi des données enregistrées lors des dernières 48 heures) ------------------------------

// ------------------- Début de la fonction envoi_info_jour (envoi des informations journalières) ---------------------------------------------------
void envoi_info_jour()
{
    byte k;

    Indic_Info_Jour = 1;
    //génération du nom de fichier et du path pour lecture des valeurs.
    memcpy(&Nom_Fichier_Rep[0], "sj", 2);
    memcpy(&Nom_Fichier_Rep[2], &Annee_Recv,4);
    memcpy(&Nom_Fichier_Rep[6], "/", 1);
    memcpy(&Nom_Fichier_Rep[7], "sj", 2);
    memcpy(&Nom_Fichier_Rep[9], &Mois_Recv,2);
    memcpy(&Nom_Fichier_Rep[11], ".dat", 4);
    Nom_Fichier_Rep[15] = '\0';
        #ifdef DEBUG_ELEC
            Serial.print(F("nom fichier pour affichage du tableau de valeur = "));
            Serial.println(Nom_Fichier_Rep);
        #endif
    LogFile = SD.open(Nom_Fichier_Rep);
        if (LogFile) {
            //Récupération des données dans le fichier année/mois sélectionné de la SDcard.
            while (LogFile.available()) {
                for (k=0; k<82; k++){
                    C_Enrlog = LogFile.read();
						if ((C_Enrlog == '\r') || (C_Enrlog == '\n')){
							C_Enrlog = LogFile.read();                  //lecture du LF ou CR éventuel.
							break;
						}
                    Enr_Log[k] = char(C_Enrlog);
                }
                Enr_Log[80] = '\0';
                //Transmission de la chaine
                EvtServer[0] = STX;
                strncpy(&EvtServer[1], NumCard, 4);
                EvtServer[5] = Separator;
                strncpy(&EvtServer[6], "TRAJO",5);                      //code de transmission d'un jour.
                EvtServer[11] = Separator;
                strncpy(&EvtServer[12], &Enr_Log[0],80);
                EvtServer[92] = ETX;
                EvtServer[93] = '\0';
                //Vérification de la connexion du client.
					if (wsServer.isConnected()) {
						wsServer.send(EvtServer,93);                    //Envoi
					}
            }
            LogFile.close();
            // transmission en cas de la trame de fin de fichier.
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "FINJO",5);                          //code trame de fin de fichier.
            EvtServer[11] = ETX;
            EvtServer[12] = '\0';
            //Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,12);                        //Envoi
				}
        }
        else {
            // transmission en cas de fichier inexistant.
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "NFIJO",5);                          //code fichier inexistant.
            EvtServer[11] = ETX;
            EvtServer[12] = '\0';
            //Vérification de la connexion du client.
                if (wsServer.isConnected()) {
					wsServer.send(EvtServer,12);                        //Envoi
				}
        }
    Indic_Info_Jour = 0;
}
// ------------------- Fin de la fonction envoi_info_jour (envoi des informations journalières) -----------------------------------------------------

// ------------------- Début de la fonction envoiObjectifElectricite (envoi des compteurs et des objectifs électriques) -----------------------------
void envoiObjectifElectricite()
{
    int Temp;

    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
    strncpy(&EvtServer[6], "PARAM",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "03", 2);
	EvtServer[14] = Separator;
	Temp = lireInt(100);                                // Année de la dernière sauvegarde
    snprintf(&EvtServer[15],5,"%4d",Temp);
    EvtServer[19] = Separator;
	Temp = lireInt(102);                                // 1ere partie Compteur électrique Heures Creuses
    snprintf(&EvtServer[20],4,"%3d",Temp);
    EvtServer[23] = Separator;
	Temp = lireInt(104);                                // 2ere partie Compteur électrique Heures Creuses
    snprintf(&EvtServer[24],4,"%3d",Temp);
    EvtServer[27] = Separator;
	Temp = lireInt(106);                                // 1ere partie Compteur électrique Heures Pleines
    snprintf(&EvtServer[28],4,"%3d",Temp);
    EvtServer[31] = Separator;
	Temp = lireInt(108);                                // 2ere partie Compteur électrique Heures Pleines
    snprintf(&EvtServer[32],4,"%3d",Temp);
    EvtServer[35] = Separator;
	Temp = EEPROM.read(112);                            // Nbre de jours EJP écoulés (lecture sur 1 octet)
    snprintf(&EvtServer[36],3,"%2d",Temp);
    EvtServer[38] = Separator;
	Temp = lireInt(113);                                // Objectif consommation électrique Heures Creuses
    snprintf(&EvtServer[39],6,"%5d",Temp);
    EvtServer[44] = Separator;
	Temp = lireInt(115);                                // Objectif consommation électrique Heures Pleines
    snprintf(&EvtServer[45],6,"%5d",Temp);
    EvtServer[50] = ETX;
    EvtServer[51] = '\0';
    //Vérification de la connexion du client.
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,51);                //Envoi
        }
        #ifdef DEBUG_ELEC
            Serial.println(EvtServer);
        #endif
}
// ------------------- Fin de la fonction envoiObjectifElectricite (envoi des compteurs et des objectifs électriques) -------------------------------

// ---------------------- Début de la fonction envoiObjectifEau (envoi des compteurs et des objectifs Eau) ------------------------------------------
void envoiObjectifEau()
{
    int Temp;

    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
    strncpy(&EvtServer[6], "PARAM",5);
	EvtServer[11] = Separator;
	strncpy(&EvtServer[12], "04", 2);
	EvtServer[14] = Separator;
	Temp = lireInt(100);                                // Année de la dernière sauvegarde
    snprintf(&EvtServer[15],5,"%4d",Temp);
    EvtServer[19] = Separator;
	Temp = lireInt(110);                                // Compteur Eau
    snprintf(&EvtServer[20],6,"%5d",Temp);
    EvtServer[25] = Separator;
	Temp = lireInt(117);                                // Objectif consommation Eau
    snprintf(&EvtServer[26],5,"%4d",Temp);
    EvtServer[30] = ETX;
    EvtServer[31] = '\0';
    //Vérification de la connexion du client.
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,31);                //Envoi
        }
        #ifdef DEBUG_ELEC
            Serial.println(EvtServer);
        #endif
}
// ---------------------- Fin de la fonction envoiObjectifEau (envoi des compteurs et des objectifs Eau) --------------------------------------------

// --------- Début de la fonction sauveObjectifEnergie (enregistre dans l'Eeprom les compteurs et les objectifs éléctriques et eau) -----------------
void sauveObjectifEnergie(char* Chaine)
{
    char Buff[] = "xxxxxx";
    int TempNbre;

    strncpy(Buff, &Chaine[15], 4);                  // Année de sauvegarde
    Buff[4] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(100, TempNbre);
    strncpy(Buff, &Chaine[20], 3);                  // 1ere partie Compteur électrique Heures Creuses
    Buff[3] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(102, TempNbre);
    strncpy(Buff, &Chaine[24], 3);                  // 2ere partie Compteur électrique Heures Creuses
    Buff[3] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(104, TempNbre);
    strncpy(Buff, &Chaine[28], 3);                  // 1ere partie Compteur électrique Heures Pleines
    Buff[3] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(106, TempNbre);
    strncpy(Buff, &Chaine[32], 3);                  // 2ere partie Compteur électrique Heures Pleines
    Buff[3] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(108, TempNbre);
    strncpy(Buff, &Chaine[36], 5);                  // Compteur Eau
    Buff[5] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(110, TempNbre);
    strncpy(Buff, &Chaine[42], 2);                  // Nbre de jours EJP écoulés
    Buff[2] = '\0';
    TempNbre = atoi(Buff);
    EEPROM.write(112, TempNbre);                    // byte sauvegarde sur 1 octet
    strncpy(Buff, &Chaine[45], 5);                  // Objectif conso. électrique Heures Creuses
    Buff[5] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(113, TempNbre);
    strncpy(Buff, &Chaine[51], 5);                  // Objectif conso. électrique Heures Pleines
    Buff[5] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(115, TempNbre);
    strncpy(Buff, &Chaine[57], 3);                  // Objectif conso. Eau
    Buff[3] = '\0';
    TempNbre = atoi(Buff);
    sauverInt(117, TempNbre);

    creMessage("03","envoiOK", 7);

}
// --------- Fin de la fonction sauveObjectifEnergie (enregistre dans l'Eeprom les compteurs et les objectifs éléctriques et eau) -------------------

// **************************************************************************************************************************************************
// ------------------------------ FIN DES FONCTIONS D'ECHANGE AVEC LE SERVEUR -----------------------------------------------------------------------
// **************************************************************************************************************************************************

