/*
*****************************************************************************************************************************************************
Projet Centrale domotique DomoMaison 2017 pour carte Arduino Mega 2560 et carte Ethernet (W5100)
Version 1.70 du 06/09/2017
Copyright 2017 PRIN Jean-Luc
Ce programme fonctionne avec le protocole Websockets (HTML5).
*****************************************************************************************************************************************************
*/
/*
*****************************************************************************************************************************************************
Modules contenant les fonctions nécessaires à la gestion de l'alarme et des caméras de surveillance
*****************************************************************************************************************************************************
*/
// *************************** Début des fonctions pour la gestion de l'alarme **********************************************************************

// -------------------------- Début de la fonction envoi_para_alarme (envoi de la configuration de l'alarme) ----------------------------------------
void envoi_para_alarme ()
{
    byte i;

    // Elaboration du message de l'état de l'alarme (avtivée ou désactivée
    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
    strncpy(&EvtServer[6], "PARAM",5);
    EvtServer[11] = Separator;
    strncpy(&EvtServer[12], "07",2);
    EvtServer[14] = Separator;
        if (Indic_Present_Absent == 1) {                // Mode présent
            if (Demande_Active_Alarme == 1) {           // demande d'activation de l'alarme en cours
                strncpy(&EvtServer[15], "2",1);         // Alarme en cours d'activation
            }
            else {
                strncpy(&EvtServer[15], "0",1);         // Alarme désactivée (mode présent)
            }
        }
        else {                                          // Mode Absent (Alarme activée)
            strncpy(&EvtServer[15], "1",1);             // Alarme activée
        }
        for (i=0; i < 10; i++){                         // envoi du mode des contacteurs (actif ou inactif)
            EvtServer[16 + (i*2)] = Separator;
			snprintf(&EvtServer[17 + (i*2)],2,"%1d",Mode_Contact[i]);
        }
        for (i=0; i < 10; i++){                         // envoi du mode des détecteurs IR (actif ou inactif)
            EvtServer[36 + (i*2)] = Separator;
			snprintf(&EvtServer[37 + (i*2)],2,"%1d",Mode_Detect[i]);
        }
        for (i=0; i < 3; i++){                         // envoi du mode des 3 zones (activé ou non)
            EvtServer[56 + (i*2)] = Separator;
			snprintf(&EvtServer[57 + (i*2)],2,"%1d",Zone_Alarme[i]);
        }
    EvtServer[62] = Separator;
        if (Indic_Effraction_Active == 1) {
            if (Mode_Silence == 1) {                    // Mode Silence activé (pas de sirène, pas de gyro selon config, envoi Email)
                strncpy(&EvtServer[63], "2",1);
            }
            else {                                      // Mode Silence non actif (Sirène, Gyro actifs, envoi Email)
                strncpy(&EvtServer[63], "1",1);
            }
        }
        else {
            strncpy(&EvtServer[63], "0",1);
        }
    EvtServer[64] = Separator;
        if (Mode_Silence == 1) {                        // Mode Silence activé (pas de sirène, pas de gyro selon config, envoi Email)
            strncpy(&EvtServer[65], "1",1);
        }
        else {
            strncpy(&EvtServer[65], "0",1);
        }
    EvtServer[66] = ETX;
    EvtServer[67] = '\0';
        #ifdef DEBUG_ALAR
            Serial.print(F("chaine emise = "));
            Serial.println(EvtServer);
        #endif
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,67);
        }
}
// -------------------------- Fin de la fonction envoi_para_alarme (envoi de la configuration de l'alarme) ------------------------------------------

// -------------------------- Début de la fonction envoi_web_alarme (envoi des informations de l'alarme au serveur ----------------------------------
void envoi_web_alarme()
{
    byte i;

    // Envoi des contacteurs
    for (i=0; i < 10; i++){
        EvtServer[0] = STX;
        strncpy(&EvtServer[1], NumCard, 4);
        EvtServer[5] = Separator;
        strncpy(&EvtServer[6], "DEFFC",5);
        EvtServer[11] = Separator;
        snprintf(&EvtServer[12],3,"%2d",i);
        EvtServer[14] = Separator;
        strncpy(&EvtServer[15], Date_Dernier_Evt_Con[i], 20);   // Date et heure du dernier déclenchement
        EvtServer[35] = ETX;
        EvtServer[36] = '\0';
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,36);            //Envoi
        }
    }
    // Envoi des détecteurs IR
    for (i=0; i < 10; i++){
        EvtServer[0] = STX;
        strncpy(&EvtServer[1], NumCard, 4);
        EvtServer[5] = Separator;
        strncpy(&EvtServer[6], "DEFFD",5);
        EvtServer[11] = Separator;
        snprintf(&EvtServer[12],3,"%2d",i);
        EvtServer[14] = Separator;
        strncpy(&EvtServer[15], Date_Dernier_Evt_Pir[i], 20);   // Date et heure du dernier déclenchement détecteur IR
        EvtServer[35] = ETX;
        EvtServer[36] = '\0';
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,36);            //Envoi
        }
    }

    // Envoi de l'état de l'alarme et de la sirène
        EvtServer[0] = STX;
        strncpy(&EvtServer[1], NumCard, 4);
        EvtServer[5] = Separator;
        strncpy(&EvtServer[6], "ETARL",5);
        EvtServer[11] = Separator;
        strncpy(&EvtServer[12], "01",2);                // non utilisé
        EvtServer[14] = Separator;
        if (Indic_Present_Absent == 1) {
            if (Demande_Active_Alarme == 1) {           // demande d'activation de l'alarme en cours
                strncpy(&EvtServer[15], "2",1);         // Alarme en cours d'activation
            }
            else {
                strncpy(&EvtServer[15], "0",1);         // Alarme désactivée (mode présent)
            }
        }
        else {                                          // Mode Absent (Alarme activée)
            strncpy(&EvtServer[15], "1",1);             // Alarme activée sans Mode Silence
        }
        EvtServer[16] = Separator;
        if (Indic_Effraction_Active == 1) {
            if (Mode_Silence == 1) {                    // Mode Silence activé (pas de sirène, pas de gyro selon config, envoi Email)
                strncpy(&EvtServer[17], "2",1);
            }
            else {                                      // Mode Silence non actif (Sirène, Gyro actifs, envoi Email)
                strncpy(&EvtServer[17], "1",1);
            }
        }
        else {
            strncpy(&EvtServer[17], "0",1);
        }
        EvtServer[18] = ETX;
        EvtServer[19] = '\0';
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,19);            //Envoi
        }

}
// -------------------------- Fin de la fonction envoi_web_alarme (envoi des informations de l'alarme au serveur ------------------------------------

// ------------------------ Début de la fonction envoi_journal_alarme (envoi des journaux de l'alarme) ----------------------------------------------
void envoi_journal_alarme(byte Journal)
{
    byte i;
    char NomFichier[] = "xxxxxxxx.dat";

    Indic_Journal_Alarme = 1;

    //génération du nom de fichier et du path pour lecture des valeurs.
        if (Journal == 15) {
			memcpy(&NomFichier[0], "alarmode.dat", 12);
		}
		else if (Journal == 16) {
			memcpy(&NomFichier[0], "alarcont.dat", 12);
		}
		else if (Journal == 17) {
			memcpy(&NomFichier[0], "alardete.dat", 12);
		}
        NomFichier[12] = '\0';

    LogFile = SD.open(NomFichier);
        if (LogFile) {
            //Récupération des données dans le fichier journal alarme demandé.
            while (LogFile.available()) {
                for (i=0; i<25; i++){
                    C_Enrlog = LogFile.read();
						if ((C_Enrlog == '\r') || (C_Enrlog == '\n')){
							C_Enrlog = LogFile.read();                  //lecture du LF ou CR éventuel.
							break;
						}
                    Enr_Log[i] = char(C_Enrlog);
                }
                Enr_Log[22] = '\0';
                //Transmission de la chaine
                EvtServer[0] = STX;
                strncpy(&EvtServer[1], NumCard, 4);
                EvtServer[5] = Separator;
                strncpy(&EvtServer[6], "JOUAL",5);                          //code de transmission d'un jour.
                EvtServer[11] = Separator;
                strncpy(&EvtServer[12], &Enr_Log[0],22);
                EvtServer[34] = ETX;
                EvtServer[35] = '\0';
                //Vérification de la connexion du client.
					if (wsServer.isConnected()) {
						wsServer.send(EvtServer,35);                        //Envoi
					}
            }
            LogFile.close();
            // transmission en cas de la trame de fin de fichier.
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "FINJO",5);                              //code trame de fin de fichier.
            EvtServer[11] = ETX;
            EvtServer[12] = '\0';
            //Vérification de la connexion du client.
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,12);                            //Envoi
				}
        }
        else {
            // transmission en cas de fichier inexistant.
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "NFIJO",5);                              //code fichier inexistant.
            EvtServer[11] = ETX;
            EvtServer[12] = '\0';
            //Vérification de la connexion du client.
                if (wsServer.isConnected()) {
					wsServer.send(EvtServer,12);                            //Envoi
				}
        }
    Indic_Journal_Alarme = 0;
}
// ------------------------ Fin de la fonction envoi_journal_alarme (envoi des journaux de l'alarme) ------------------------------------------------

// ------------------------- Début de la fonction traiteAlarme (traitement de la réception des capteurs d'alarme par RFLink -------------------------
void traiteAlarme(char* Chaine)
{
    char Id[] = "xxxxxx";
    byte NumZone;

        if (strncmp(&Chaine[6],"Chuango",7) == 0) {             // Réception capteur marque Chuango
            strncpy(Id, &Chaine[17],6);
        }
        else if (strncmp(&Chaine[6],"Xcellent",8) == 0) {       // Réception capteur marque Xcellent
            strncpy(Id, &Chaine[18],6);
        }

    for (byte i = 0; i < 10; i++) {
        NumZone = ZONE_CONTACT[i];                              // numéro de la zone auquel le contacteur est rattaché
            if (strcmp(Id, ID_CONTACTEUR[i]) == 0) {            // Id valide
                if ((Indic_Present_Absent == 0) && (Mode_Contact[i] > 0) && (Zone_Alarme[NumZone - 1] == 1)) {    // alarme sur absent, contacteur actif ou silence
                    if (NumZone == 1) {                         // Zone 1 temporisée afin de permetrre l'arrêt de l'alarme
                        if (DeclenAlarme == 0) {                // pas de déclenchement temporisé alarme mémorisé (afin d'éviter une réinitialisation du compteur)
                            TempsAlarmeDeclen = TempsMn;        // Heure du déclenchement en minites
                            DeclenAlarme = 1;
                            NumCapteur = i;
                            TypeCapteur = 'C';
                        }
                    }
                    else{
                        declenchement_contacteur(i);
                    }
                }
            }
        NumZone = ZONE_DETECT[i];                                   // numéro de la zone auquel le détecteur IR est rattaché
            if(strcmp(Id, ID_DETECTEUR[i]) == 0) {                  // Id valide
                if ((Indic_Present_Absent == 0) && (Mode_Detect[i] > 0) && (Zone_Alarme[NumZone - 1] == 1)) {     // alarme sur absent, détecteur actif ou Silence
                    if (NumZone == 1) {                             // Zone 1 temporisée afin de permetrre l'arrêt de l'alarme
                        if (DeclenAlarme == 0) {                    // pas de déclenchement temporisé alarme mémorisé (afin d'éviter une réinitialisation du compteur)
                            TempsAlarmeDeclen = TempsMn;            // Heure du déclenchement en minites
                            DeclenAlarme = 1;
                            NumCapteur = i;
                            TypeCapteur = 'D';
                        }
                    }
                    else {
                        declenchement_detecteur(i);
                    }
                }
            }
    }
}
// ------------------------- Fin de la fonction traiteAlarme (traitement de la réception des capteurs d'alarme par RFLink ---------------------------

// ---------------------- Début de la fonction declenchement_contacteur (validation du déclenchement par contacteur d'ouverture) --------------------
void declenchement_contacteur(byte Num)
{
    Indic_Alarme_Ouvert = 1;                                            //indicateur de détection d'une ouverture
    Indic_Effraction_Active = 1;                                        // indique qu'une effraction est en cours
    TempsEffractionActive = TempsMn;                                    // Initialisation de la durée de fonctionnement de l'effraction
    marche_gyro_sirene(Num);                                               // commande le gyrophare et la sirène
    Num_Indic_Con = Num;                                                // numéro du contacteur ouverture déclenché en dernier
    strncpy(Date_Dernier_Evt_Con[Num], date_alarme(), 20);

    enregJournalAlarme("ALAR_C", Num);                                  // Enregistrement de l'evènement dans le journal
    envoi_declenchement_alarme('C', Num);
        #ifdef DEBUG_ALAR
            Serial.print(F("Effraction sur contacteur d'ouverture "));
            Serial.print(Num);
            Serial.print(F(" le "));
            Serial.println(Date_Dernier_Evt_Con[Num]);
        #endif

}
// ---------------------- Fin de la fonction declenchement_contacteur (validation du déclenchement par contacteur d'ouverture) ----------------------

// ---------------------- Début de la fonction declenchement_detecteur (validation du déclenchement par détecteur IR) -------------------------------
void declenchement_detecteur(byte Num)
{
    Indic_Alarme_PIR = 1;                                       // indicateur de détection IR
    Indic_Effraction_Active = 1;                                // indique qu'une effraction est en cours
    TempsEffractionActive = TempsMn;                            // Initialisation de la durée de fonctionnement de l'effraction
    marche_gyro_sirene(Num);                                       // commande le gyrophare et la sirène
    Num_Indic_Pir = Num;                                        // numéro du détecteur IR déclenché en dernier

    strncpy(Date_Dernier_Evt_Pir[Num], date_alarme(), 20);

    enregJournalAlarme("ALAR_D", Num);                          // Enregistrement de l'evènement dans le journal
    envoi_declenchement_alarme('D', Num);
        #ifdef DEBUG_ALAR
            Serial.print(F("Effraction sur detecteur PIR "));
            Serial.print(Num);
            Serial.print(F(" le "));
            Serial.println(Date_Dernier_Evt_Pir[Num]);
        #endif
}
// ---------------------- Fin de la fonction declenchement_detecteur (validation du déclenchement par détecteur IR) ---------------------------------

// ---------------------- Début de la fonction envoi_declenchement alarme (envoiau les déclenchements clavier déporté) ------------------------------
void envoi_declenchement_alarme(char capteur, byte Num)
{

    char Message[] = "xx;xxxxx;xxxx;xx/xx/xxxx;xx:xx";
    byte i;

        if (capteur == 'C') {                               // Déclenchement par Contacteur ouverture
            strncpy(&Message[0], "60;NRFrl;DECO;",14);
            strncpy(&Message[14], Date_Dernier_Evt_Con[Num], 16);
        }
        else if (capteur == 'D') {                          // Déclenchement par Détecteur IR
            strncpy(&Message[0], "60;NRFrl;DEDE;",14);
            strncpy(&Message[14], Date_Dernier_Evt_Pir[Num], 16);
        }
    Message[30] = '\0';

    Serial2.println(Message);                                   // Envoi à RFLink
}
// ---------------------- Fin de la fonction envoi_declenchement alarme (envoiau les déclenchements clavier déporté) ------------------------------

// ------------------ Début de la fonction date_alarme (élaboration de la chaine contenant la date et l'heure du déclenchement de l'alarme ----------
char* date_alarme()
{
    char LigneChar[] = "xx/xx/xxxx;xx:xx:xx";

    // Elaboration de la date
    snprintf(&LigneChar[0],3,"%02d",DateJour);
    LigneChar[2] = '/';
    snprintf(&LigneChar[3],3,"%02d",Mois);
    LigneChar[5] = '/';
    snprintf(&LigneChar[6],5,"%4d",Annee);
    LigneChar[10] = Separator;

    // Elaboration de l'heure
    snprintf(&LigneChar[11],3,"%02d",Hours);
    LigneChar[13] = ':';
    snprintf(&LigneChar[14],3,"%02d",Mins);
    LigneChar[16] = ':';
    snprintf(&LigneChar[17],3,"%02d",Sec);
    LigneChar[19]= '\0';

    return LigneChar;

}
// ------------------ Fin de la fonction date_alarme (élaboration de la chaine contenant la date et l'heure du déclenchement de l'alarme) -----------

// ------------------- Début de la fonction activeAlarme (activation ou désactivation de l'alarme, demande Web) -------------------------------------
void activeAlarme(char* Chaine, char Dem)
{

        if (strncmp(&Chaine[15],"0",1) == 0) {
            Indic_Present_Absent = 1;                               // Alarme désactivée
            Demande_Active_Alarme = 0;                              // Coupe la demande d'activation de l'alarme s'il y en a une
            Mode_Silence = 0;                                       // Désactive mode silence si enclenché
            DeclenAlarme = 0;                                       // désactive les déclenchements temporisés
            TempsAlarmeActivation = 0;
            Indic_Effraction_Active = 0;                                // indique la coupure de la sirène et du gyro s'ils sont actifs
            coupe_gyro_sirene();                                    // coupe le gyrophare et la sirène s'ils sont actifs
                if (Dem == 'W') {
                    creMessage("01","désactivée", 12);              // + 2 pour caractères é accentué
                }
            enregJournalAlarme("ALAR_M", 0);                        // Enregistrement dans le journal de la désactivation de l'alarme (0 pour désactivation)
        }
        else if (strncmp(&Chaine[15],"1",1) == 0) {
            Demande_Active_Alarme = 1;                              // Demande d'activation de l'alarme (lance le décompte de la tempo d'activation)
            Mode_Silence = 0;                                       // Désactive mode silence si enclenché
            TempsAlarmeActivation = TempsMn;                        // Initialisation de l'heure de la demande pour temporisation
                if (Dem == 'W') {
                    creMessage("01","demande", 7);
                }
            enregJournalAlarme("ALAR_M", 1);                         // Enregistrement dans le journal de l'activation de l'alarme (1 pour activation)
        }
        else if (strncmp(&Chaine[15],"2",1) == 0) {
            Demande_Active_Alarme = 1;                              // Demande d'activation de l'alarme (lance le décompte de la tempo d'activation)
            Mode_Silence = 1;                                       // Demande d'activation du mode silence (pas de sirène, pas de gyro selon config, envoi mail
            TempsAlarmeActivation = TempsMn;                        // Initialisation de l'heure de la demande pour temporisation
                if (Dem == 'W') {
                    creMessage("01","demande", 7);
                }
            enregJournalAlarme("ALAR_M", 2);                         // Enregistrement dans le journal de l'activation de l'alarme (1 pour activation)
        }

        for (byte i=0; i < 3; i++) {
            if (strncmp(&Chaine[17 + (i*2)],"0",1) == 0) {          // désactivation de la zone
                Zone_Alarme[i] = 0;
            }
            else {
                Zone_Alarme[i] = 1;
            }
        }
    envoi_etatAlarme();                                                   // Envoi au clavier déporté l'état de l'alarme
}
// ---------------------------- Fin de la fonction activeAlarme (activation ou désactivation de l'alarme, demande web) -------------------------------

// ----------------- Début de la fonction activeContactA (activation ou désactivation des contacteurs d'ouverture de l'alarme) ----------------------
void activeContactA(char* Chaine)
{
    for (byte i=0; i < 10; i++){
        if (strncmp(&Chaine[15 + (i*2)],"0",1) == 0) {              // désactivation du contacteur
            Mode_Contact[i] = 0;
            EEPROM.write(46 + i,0);
        }
        else if (strncmp(&Chaine[15 + (i*2)],"1",1) == 0) {
            Mode_Contact[i] = 1;                                    // activation du contacteur
            EEPROM.write(46 + i,1);
        }
        else if (strncmp(&Chaine[15 + (i*2)],"2",1) == 0) {
            Mode_Contact[i] = 2;                                    // activation du contacteur en mode Silence (pas de sirène)
            EEPROM.write(46 + i,2);
        }
    }
    creMessage("02","contacteurs OK", 14);
}
// ----------------- Fin de la fonction activeContactA (activation ou désactivation des contacteurs d'ouverture de l'alarme) ------------------------

// ----------------- Début de la fonction activeDetectA (activation ou désactivation des détecteurs IR de l'alarme) ---------------------------------
void activeDetectA(char* Chaine)
{
    for (byte i=0; i < 10; i++){
        if (strncmp(&Chaine[15 + (i*2)],"0",1) == 0) {              // désactivation du détecteur
            Mode_Detect[i] = 0;
            EEPROM.write(56 + i,0);
        }
        else if (strncmp(&Chaine[15 + (i*2)],"1",1) == 0) {
            Mode_Detect[i] = 1;                                     // activation du détecteur
            EEPROM.write(56 + i,1);
        }
        else if (strncmp(&Chaine[15 + (i*2)],"2",1) == 0) {
            Mode_Detect[i] = 2;                                     // activation du détecteur
            EEPROM.write(56 + i,2);
        }
    }
    creMessage("03","détecteurs OK", 14);                           // +1 pour caractère é accentuée
}
// ----------------- Fin de la fonction activeDetectA (activation ou désactivation des détecteurs IR de l'alarme) -----------------------------------

// ------------------ Début de la fonction marche_gyro_sirene (marche en fonctionnement du gyrophare et de la sirène) -------------------------------
void marche_gyro_sirene(byte Num)
{
        if (Mode_Silence == 0) {                    // pas de mode Silence actif
            if (Mode_Detect[Num] == 1) {            // pas de mode Silence au niveau du capteur
                // la mise en marche de la sirène s'effectue par une mise en niveau bas des broches (à adapter à votre sirène si nécessaire)
                digitalWrite(PIN_SIRENE,LOW);
                //delay(1000);                          // Impulsion d'une seconde
                //digitalWrite(PIN_SIRENE,HIGH);
            }
            else {
            digitalWrite(PIN_SIRENE,HIGH);          // Coupure Sirène en cas de fonctionnement
            }
        }

        if (MODE_GYRO == "ON") {                   // Commande du gyro meme en mode Silence
            // Commande du gyrophare
            digitalWrite(PIN_GYRO,LOW);
        }
        else {
            digitalWrite(PIN_GYRO,HIGH);            // Coupure du gyro en cas de fonctionnement
        }

        #ifdef DEBUG_ALAR
            Serial.println(F("Mise en marche Effraction"));
        #endif
}
// ------------------ Fin de la fonction marche_gyro_sirene (marche en fonctionnement du gyrophare et de la sirène) ---------------------------------

// ---------------------------- Début de la fonction coupe_gyro_sirene (arrêt du gyrophare et de la sirène) -----------------------------------------
void coupe_gyro_sirene()
{
    // remise en niveau bas
    digitalWrite(PIN_SIRENE,HIGH);
    // arrêt du gyrophare
    digitalWrite(PIN_GYRO,HIGH);

        #ifdef DEBUG_ALAR
            Serial.println(F("Coupure Effraction"));
        #endif
}
// ---------------------------- Début de la fonction coupe_gyro_sirene (arrêt du gyrophare et de la sirène) -----------------------------------------

// ------------------ Début de la fonction enregJournalAlarme (enregistrement des évènements de l'alarme dans les journaux) -------------------------
void enregJournalAlarme(char* Action, byte Posi)
{
    char ChaineJ[] = "xx/xx/xxxx;xx:xx:xx;xx";
    char NomFichier[] = "xxxxxxxx.dat";

    //horodate.
    strncpy(ChaineJ, date_alarme(), 19);                         // Date de l'évènement
    ChaineJ[19] = Separator;

        if (strncmp(Action,"ALAR_M",6) == 0) {
			memcpy(&NomFichier[0], "alarmode.dat", 12);
                switch (Posi)
                {
                    case 0:
                        strncpy(&ChaineJ[20], "OF",2);
                        break;
                    case 1:
                        strncpy(&ChaineJ[20], "ON",2);
                        break;
                    case 2:
                        strncpy(&ChaineJ[20], "OS",2);
                        break;
                }
		}
        else if (strncmp(Action,"ALAR_C",6) == 0) {
			memcpy(&NomFichier[0], "alarcont.dat", 12);
			snprintf(&ChaineJ[20],3,"%02d",Posi);
		}
        else if (strncmp(Action,"ALAR_D",6) == 0) {
			memcpy(&NomFichier[0], "alardete.dat", 12);
			snprintf(&ChaineJ[20],3,"%02d",Posi);
		}
    ChaineJ[22] = '\0';
    NomFichier[12]='\0';

        if (!SD.exists(NomFichier)) {
            LogFile = SD.open(NomFichier, FILE_WRITE);
            LogFile.close();
            //Controle de la creation du fichier.
                if (SD.exists(NomFichier)){
				}
        }

    //Enregistrement dans la SDcard
    LogFile = SD.open(NomFichier, FILE_WRITE);
        if (LogFile) {
            LogFile.println(ChaineJ);
            LogFile.close();
        }
}
// ------------------ Fin de la fonction enregJournalAlarme (enregistrement des évènements de l'alarme dans les journaux) ---------------------------

// ------------------ Début de la fonction effaceJournalAlarme (effacement des journaux de l'alarme) ------------------------------------------------
void effaceJournalAlarme(char* Journal)
{
    byte i;
    boolean Tous = false;

        if (strncmp(Journal,"MO",2) == 0) {
            SD.remove("alarmode.dat");
        }
        else if (strncmp(Journal,"CO",2) == 0) {
            SD.remove("alarcont.dat");
                for (i = 0; i < 10; i++) {
                    strncpy(Date_Dernier_Evt_Con[i],"xx/xx/xxxx;xx:xx:xx", 20);
                }
        }
        else if (strncmp(Journal,"DE",2) == 0) {
            SD.remove("alardete.dat");
                for (i = 0; i < 10; i++) {
                    strncpy(Date_Dernier_Evt_Pir[i],"xx/xx/xxxx;xx:xx:xx", 20);
                }
        }
        else if (strncmp(Journal,"TO",2) == 0) {
            SD.remove("alarmode.dat");
            SD.remove("alarcont.dat");
            SD.remove("alardete.dat");
                for (i = 0; i < 10; i++) {
                }
                    strncpy(Date_Dernier_Evt_Con[i],"xx/xx/xxxx;xx:xx:xx", 20);
                    strncpy(Date_Dernier_Evt_Pir[i],"xx/xx/xxxx;xx:xx:xx", 20);
            Tous = true;
       }

       if (Tous == false) {
            creMessage("01","Effacement du journal en cours effectué", 40);     // +1 pour caractète é accentué
       }
       else {
            creMessage("01","Effacement de tous les journaux effectué", 41);    // +1 pour caractète é accentué
       }

}
// ------------------ Fin de la fonction effaceJournalAlarme (effacement des journaux de l'alarme) --------------------------------------------------

// ------------------------- DEBUT DES FONCTIONS POUR LE CLAVIER DEPORTE ----------------------------------------------------------------------------
// ---------------------- Début de la fonction etatAlarme (Etat de l'alarme pour envoi au clavier déporté) ------------------------------------------
void envoi_etatAlarme()
{
    char Message[] = "xx;xxxx;xxxx;xxxx";
    byte i;

    strncpy(&Message[0], "60;NRFrl;ETAT;",14);

        if (Indic_Present_Absent == 1) {
            if (Demande_Active_Alarme == 1) {           // demande d'activation de l'alarme en cours
                if (Mode_Silence == 1) {                // Mode Silence activé
                    Message[14] = '4';                  // Alarme en cours d'activation
                }
                else {                                  // Mode Silence non activé (=0)
                    Message[14] = '2';                  // Alarme en cours d'activation
                }
            }
            else {
                Message[14] = '0';                      // Alarme désactivée (mode présent)
            }
        }
        else {                                          // Mode Absent (Alarme activée)
                if (Mode_Silence == 1) {                // Mode Silence activé
                    Message[14] = '3';                  // Alarme activée avec Mode Silence
                }
                else {                                  // Mode Silence non activé (=0)
                    Message[14] = '1';                  // Alarme activée sans Mode Silence
                }
        }

        for (i=0; i < 3; i++){                                  // envoi du mode des 3 zones (activé ou non)
			snprintf(&Message[15 + i],2,"%1d",Zone_Alarme[i]);
        }
    Message[18] = '\0';

    Serial2.println(Message);                                   // Envoi à RFLink

}
// ---------------------- Fin de la fonction etatAlarme (Etat de l'alarme pour envoi au clavier déporté) --------------------------------------------
// ---------------------- Fin de la gestion de l'alarme ---------------------------------------------------------------------------------------------

// ---------------------- Début de la gestion des caméras -------------------------------------------------------------------------------------------
// ------------------------- Début de la fonction envoi_etat_cameras (Envoi les états de fonctionnement des caméras) --------------------------------
void envoi_etat_cameras()
{
    // Elaboration du message de l'état de fonctionnement des caméras
    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
    strncpy(&EvtServer[6], "PARAM",5);
    EvtServer[11] = Separator;
    strncpy(&EvtServer[12], "14",2);
    EvtServer[14] = Separator;
        if (EtatIntrusion == 1) {                       // Détection INtrusion activée
                strncpy(&EvtServer[15], "1",1);
            }
            else {
                strncpy(&EvtServer[15], "0",1);
            }
    EvtServer[16] = Separator;
        if (EtatEnreg == 1) {                       // Détection INtrusion activée
                strncpy(&EvtServer[17], "1",1);
            }
            else {
                strncpy(&EvtServer[17], "0",1);
            }
    EvtServer[18] = Separator;
        if (EtatPir == 1) {                       // Détection INtrusion activée
                strncpy(&EvtServer[19], "1",1);
            }
            else {
                strncpy(&EvtServer[19], "0",1);
            }
    EvtServer[20] = ETX;
    EvtServer[21] = '\0';
        #ifdef DEBUG_ALAR
            Serial.print(F("chaine emise = "));
            Serial.println(EvtServer);
        #endif
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,21);
        }
}
// ------------------------- Fin de la fonction envoi_etat_cameras (Envoi les états de fonctionnement des caméras) ----------------------------------

// ------------------------- Début de la fonction modif_ete_cameras (Modifie les états de fonctionnement des caméras) -------------------------------
void modif_etat_cameras(char* Chaine)
{
        if (strncmp(&Chaine[15],"0",1) == 0) {
            EtatIntrusion = 0;                      // Détection Intrusion non activée
        }
        else {
            EtatIntrusion = 1;                      // Détection Intrusion activée
        }
        if (strncmp(&Chaine[17],"0",1) == 0) {
            EtatEnreg = 0;                          // Enregistrement programmé non activée
        }
        else {
            EtatEnreg = 1;                          // Enregistrement programmé activée
        }
        if (strncmp(&Chaine[19],"0",1) == 0) {
            EtatPir = 0;                            // Enregistrement sur capteur PiR non activée
        }
        else {
            EtatPir = 1;                            // Enregistrement sur capteur PIR activée
        }

    // Enregistrement des états dans l'EEPROM
    EEPROM.write(66, EtatIntrusion);
    EEPROM.write(67, EtatEnreg);
    EEPROM.write(68, EtatPir);

    // Envoi des états de fonctionnement des caméras à l'application Web
    envoi_etat_cameras();

    // Envoi du messge de bonne réception des modification
    creMessage("01","Config OK", 9);
}
// ------------------------- Fin de la fonction modif_ete_cameras (Modifie les états de fonctionnement des caméras) ---------------------------------
// ---------------------- Fin de la gestion des caméras -------------------------------------------------------------------------------------------

