/*
*****************************************************************************************************************************************************
Projet Centrale domotique DomoMaison 2017 pour carte Arduino Mega 2560 et carte Ethernet (W5100)
Version 1.70 du 06/09/2017
Copyright 2017 PRIN Jean-Luc
Ce programme fonctionne avec le protocole Websockets (HTML5).
*****************************************************************************************************************************************************
*/
/*
*****************************************************************************************************************************************************
Modules contenant les fonctions nécessaires à la gestion des volets, du portail et des portes de garage, ainsi que de leur programmation
Comprend également les fonctions pour la commande des prises éléctriques
*****************************************************************************************************************************************************
*/
// *************************** Début des fonctions pour la commande des volets **********************************************************************

// -------------------------- Début de la fonction envoiParametres Volets ---------------------------------------------------------------------------
void envoiParametresVolets()
{
    byte Temp;

    // Elaboration du 1er message (Durée de fonctionnement des volets)
    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
    strncpy(&EvtServer[6], "PARAM",5);
    EvtServer[11] = Separator;
    strncpy(&EvtServer[12], "01",2);                            // 01 pour signaler envoi paramètres des volets
        for (int i=0; i < 10; i++){
            EvtServer[14 + (i*3)] = Separator;
            Temp = EEPROM.read(i);
			snprintf(&EvtServer[15 + (i*3)],3,"%02d",Temp);     // le numéro de la pièce commence à 1
        }
    EvtServer[44] = ETX;
    EvtServer[45] = '\0';
        #ifdef DEBUG_TELECOM
            Serial.print(F("chaine emise = "));
            Serial.println(EvtServer);
        #endif
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,45);//Envoi
        }
}
// -------------------------- Fin de la fonction envoiParametresVolets ------------------------------------------------------------------------------

// -------------------- Début de la procédure de sauvegarde des paramètres d'initialisation des volets ----------------------------------------------
void sauveParametresVolets(char* Chaine)
{
    byte Temps;
    char Buff[3];

        #ifdef DEBUG_TELECOM
            Serial.print(F("chaine recue = "));
            Serial.write(Chaine);
            Serial.println();
        #endif

        for (byte i=0; i < 10; i++){
            Buff[0] = Chaine[15 + (i*3)];
            Buff[1] = Chaine[16 + (i*3)];
            Buff[2] = '\0';
            Temps = atoi(Buff);
            EEPROM.write(i, Temps);
        }

    creMessage("01", "envoiOK", 7);
}
// -------------------- Fin de la procédure de sauvegarde des paramètres de la centrale de gestion --------------------------------------------------

// -------------------------- Début de la fonction gestionVolets ------------------------------------------------------------------------------------
void gestion_volets(char* Chaine)
{
    byte ModeSelect;                                                            // 0 pour simple (volets sélectionnés), 1 pour multiple (tous les volets)
    byte VoletDepart;                                                           // volet de départ au niveau de la télécommande
    char Buff[4];
    byte Action;                                                                 // Action en chiffre entier pour la fonction switch
    byte Pourcen;                                                                // Pourcentage de fermeture des volets
    byte Tps;                                                                    // durée de fonctionnement des volets pour mouvement à 100%
    int Duree;
    byte i;                                                                  // durée de fonctionnement du volet au pourcentage demandée

    Buff[0] = Chaine[15];
    Buff[1] = Chaine[16];
    Buff[2] = Chaine[17];
    Buff[3] = '\0';
    Pourcen = atoi(Buff);
    Buff[0] = '0';
    Buff[1] = Chaine[12];
    Buff[2] = Chaine[13];
    Buff[3] = '\0';
    Action = atoi(Buff);

        if (Pourcen!=100) {                                                     // si pourcentage différent de 100%, Ouvre d'abord le volet, puis le referme au % demandé
            Action = 7;
        }

        #ifdef DEBUG_TELECOM
            Serial.print(F("Action demandee :"));
            Serial.println(Action);
            Serial.println(Pourcen);
        #endif

    switch (Action) {
        case 1:                                                                 // Ouverture des volets séléctionnés
            ModeSelect = EEPROM.read(0);                                        // extraction de la position du mode sur la télécommande mis en mémoire dans l'Eeprom
            VoletDepart = EEPROM.read(1);                                       // extraction du volet de départ sur la télécommande mis en mémoire dans l'Eeprom
            creMessage("01", "Ouverture volets selectionnes", 29);
                if (ModeSelect == 1) {
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie court sur bouton du milieu (Sélection Mode Simple)
                }
            EEPROM.write(0,0);                                                  // mise en mémoire dans l'Eeprom du mode Simple
                if (VoletDepart != 1) {
                    for (i=0; i < (VoletDepart - 1); i++){
                        appuiTouche(PIN_VOL_GAUCHE);                                 // Appuie sur bouton de gauche pour amener sur la sélection du premier volet
                    }
                }
                for (i = 0; i < 8; i++){
                    if (strncmp(&Chaine[19 + i],"S",1) == 0){
                        appuiTouche(PIN_VOL_HAUT);                                   // Appuie sur bouton Cde Haut (Ouvrir)
                    }
                    appuiTouche(PIN_VOL_DROIT);                                      // Appuie sur bouton Cde droit pour changer de volet
                }
            EEPROM.write(1,1);                                                  // mise en mémoire dans l'Eeprom dela position de départ (volet n°1)
            break;
        case 2:                                                                 // Ouverture de tous les volets
            ModeSelect = EEPROM.read(0);
            creMessage("01", "Ouverture de tous les volets", 28);
                if (ModeSelect ==0) {
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie sur bouton du milieu (Sélection Mode Multiple)
                }
            EEPROM.write(0,1);                                                  // mise en mémoire dans l'Eeprom du mode Multiple
            appuiTouche(PIN_VOL_HAUT);                                               // Appuie sur bouton Cde Haut (Ouvrir)
            break;
        case 3:                                                                 // Arrêt des volets séléctionnés
            ModeSelect = EEPROM.read(0);                                        // extraction de la position du mode sur la télécommande mis en mémoire dans l'Eeprom
            VoletDepart = EEPROM.read(1);                                       // extraction du volet de départ sur la télécommande mis en mémoire dans l'Eeprom
            creMessage("01", "Arrêt des volets selectionnes", 30);
                if (ModeSelect == 1) {
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie sur bouton du milieu (Sélection Mode Simple)
                }
            EEPROM.write(0,0);                                                  // mise en mémoire dans l'Eeprom du mode Simple
                if (VoletDepart != 1) {
                    for (i=0; i < (VoletDepart -1); i++){
                        appuiTouche(PIN_VOL_GAUCHE);                                 // Appuie sur bouton de gauche pour amener sur la sélection du premier volet
                    }
                }
                for (i=0; i < 8; i++){
                    if (strncmp(&Chaine[19 + i],"S",1) == 0){
                        appuiTouche(PIN_VOL_STOP);                                   // Appuie sur bouton Cde Stop (Arrêter)
                    }
                    appuiTouche(PIN_VOL_DROIT);                                      // Appuie sur bouton Cde droit pour changer de volet
                }
            EEPROM.write(1,1);                                                  // mise en mémoire dans l'Eeprom dela position de départ (volet n°1)
            break;
        case 4:                                                                 // Arrêt de tous les volets
            ModeSelect = EEPROM.read(0);
            creMessage("01", "Arrêt de tous les volets", 25);
                if (ModeSelect == 0) {                                           // se met en mode multiple pour arreter tous les volets
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie sur bouton du milieu (Sélection Mode Multiple)
                }
            EEPROM.write(0,1);                                                  // mise en mémoire dans l'Eeprom du mode Multiple
            appuiTouche(PIN_VOL_STOP);                                               // Appuie sur bouton Stop
            break;
        case 5:                                                                 // fermeture des volets séléctionnés
            ModeSelect = EEPROM.read(0);
            VoletDepart = EEPROM.read(1);
            creMessage("01", "Fermeture des volets selectionnes", 33);
                if (ModeSelect == 1) {
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie sur bouton du milieu (Sélection Mode Simple)
                }
            EEPROM.write(0,0);                                                  // mise en mémoire dans l'Eeprom du mode Simple
                if (VoletDepart != 1) {
                    for (i=0; i < (VoletDepart -1); i++){
                        appuiTouche(PIN_VOL_GAUCHE);                                 // Appuie sur bouton de gauche pour amener sur la sélection du premier volet
                    }
                }
                for (i=0; i < 8; i++){
                    if (strncmp(&Chaine[19 + i],"S",1) == 0){
                        appuiTouche(PIN_VOL_BAS);                                    // Appuie sur bouton du bas pour fermeture des volets sélectionnés
                    }
                    appuiTouche(PIN_VOL_DROIT);                                      // Appuie sur bouton Cde droit pour changer de volet
                }
            EEPROM.write(1,1);                                                  // mise en mémoire dans l'Eeprom dela position de départ (volet n°1)
            break;
        case 6:                                                                 // fermeture de tous les volets
            ModeSelect = EEPROM.read(0);
            creMessage("01", "Fermeture de tous les volets", 28);
                if (ModeSelect == 0) {
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie sur bouton du milieu (Sélection Mode Multiple)
                }
            EEPROM.write(0,1);                                                  // mise en mémoire dans l'Eeprom du mode Multiple
            appuiTouche(PIN_VOL_BAS);                                                // Appuie sur bouton du bas pour fermeture des volets sélectionnés
            break;
        case 7:
            ModeSelect = EEPROM.read(0);                                        // extraction de la position du mode sur la télécommande mis en mémoire dans l'Eeprom
            VoletDepart = EEPROM.read(1);                                       // extraction du volet de départ sur la télécommande mis en mémoire dans l'Eeprom
            creMessage("01", "Volets en pourcentage selectionnes", 34);
                if (ModeSelect == 1) {
                    appuiTouche(PIN_VOL_MILIEU);                                     // Appuie court sur bouton du milieu (Sélection Mode Simple)
                }
            EEPROM.write(0,0);                                                  // mise en mémoire dans l'Eeprom du mode Simple
                if (VoletDepart != 1) {
                    for (i=0; i < (VoletDepart -1); i++){
                        appuiTouche(PIN_VOL_GAUCHE);                                 // Appuie sur bouton de gauche pour amener sur la sélection du premier volet
                    }
                }
                for (i=0; i < 8; i++){
                    if (strncmp(&Chaine[19 + i],"S",1) == 0){
                        Tps = EEPROM.read(i + 2);                                 // lit la durée de fonctionnement du volet pour 100% dans l'Eeprom
                        Duree = Tps * Pourcen * 10;                              // durée de fonctionnement de volet demandée
                        Tps = Tps * 1000;
                        appuiTouche(PIN_VOL_HAUT);                                   // Appuie sur bouton Cde Haut (Ouvrir)
                        delay(Tps);                                             // attends que le volet soit ouvert complètement
                        appuiTouche(PIN_VOL_BAS);                                    // fait descendre le volet
                        delay(Duree);                                           // attends le temps correspondant au pourcentage
                        appuiTouche(PIN_VOL_STOP);                                   //  arrête le volet
                    }
                    appuiTouche(PIN_VOL_DROIT);                                      // Appuie sur bouton Cde droit pour changer de volet
                }
            EEPROM.write(1,1);                                                  // mise en mémoire dans l'Eeprom dela position de départ (volet n°1)
            break;
        }
}
// -------------------------- Fin de la fonction gestionVolets --------------------------------------------------------------------------------------

// ---------- Début de la fonction charger_prog_volet (chargement de la programmation des volets pour la journée) -----------------------------------
void charger_prog_volet()
{
    byte Temp;
    boolean Act;
    byte JourProg;
    byte MoisProg;
    byte AnneeProg;
    byte HreTemp;
    byte MinTemp;
    byte Inc=0;
    boolean Jok;
    NbProgV=0;

        for (int i=0; i<NB_VOLETS; i++) {
            Act = EEPROM.read(1000 + (i*24));                                               // activation du programme
                if (Act==1) {                                                               // programme activé
                    JourProg = EEPROM.read(1001 + (i*24));                                  // jour de la programmation (1 à 31)
                    MoisProg = EEPROM.read(1002 + (i*24));                                  // mois du programme
                    AnneeProg = EEPROM.read(1003 + (i*24));                                 // annee du programme (sans le millésime)
                    // recherche d'abord si la journée courante est concernée par une programmation des volets
                    Jok = false;
                        if ((JourCourant == 6 || JourCourant == 7) && JourProg == 33) {                         // 33 = wek-end
                            Jok = true;
                        }
                        else if ((JourCourant > 0 && JourCourant <6 ) && JourProg == 32) {                      // 32 = semaine
                            Jok = true;
                        }
                        else if ((JourProg == DateJour) && (MoisProg == Mois) && ((AnneeProg + 2000) == Annee)) {
                            Jok = true;
                        }
                        if (Jok == true) {
                            NumProg[Inc] = i + 1;
                            Temp = EEPROM.read(1020 + (i*24));
                                if (Temp != 99) {
                                    HreTemp = Temp;
                                    Temp = EEPROM.read(1021 + (i*24));
                                    MinTemp = Temp;
                                    HreOu[Inc] = (HreTemp * 60) + MinTemp;
                                }
                                else {
                                    HreOu[Inc] = 1500;
                                }
                            Temp = EEPROM.read(1022 + (i*24));
                                if (Temp != 99) {
                                    HreTemp = Temp;
                                    Temp = EEPROM.read(1023 + (i*24));
                                    MinTemp = Temp;
                                    HreFe[Inc] = (HreTemp * 60) + MinTemp;
                                }
                                else {
                                    HreFe[Inc] = 1500;
                                }
                            #ifdef DEBUG_TELECOM
                                Serial.print(F("Prog = "));
                                Serial.print(NumProg[Inc]);
                                Serial.print(F(";"));
                                Serial.print(HreOu[Inc]);
                                Serial.print(F(";"));
                                Serial.println(HreFe[Inc]);
                            #endif
                        Inc = Inc + 1;
                    }
                }
        }
    NbProgV = Inc;
}
// ---------- Fin de la fonction charger_prog_volet (chargement de la programmation des volets pour la journée) -------------------------------------

// ----------------- Début de la fonction application de la programmation des volets ----------------------------------------------------------------
void applique_prog_volets()
{
    byte Prog;
    char Ordre;

        if (NbProgV > 0) {                                          // si au moins un programme pour les volets
            for (int i=0; i < NbProgV; i++){
                if (HreOu[i] <= TempsMn) {
                    Prog = NumProg[i];
                    #ifdef DEBUG_TELECOM
                        Serial.println(F("Ouverture volet"));
                    #endif
                    Ordre = 'O';                                    // ouverture volets programmés
                    HreOu[i] = 1500;                                // permet de considérer l'action effectuée
                    action_prog_volet(Prog, Ordre);
                }
                if (HreFe[i] <= TempsMn) {
                    Prog = NumProg[i];
                    #ifdef DEBUG_TELECOM
                        Serial.println(F("Fermeture volet"));
                    #endif
                    Ordre = 'F';                                    // fermeture volets programmés
                    HreFe[i] = 1500;                                // permet de considérer l'action effectuée
                    action_prog_volet(Prog, Ordre);
                }
            }
        }
}
// ----------------- Fin de la fonction application de la programmation des volets ------------------------------------------------------------------

// ----------------- Début de la fonction action_prog_volet (ouverture ou fermeture programmée des volets) ------------------------------------------
void action_prog_volet(byte Prog, char Action)
{
    byte ModeSelect;                                        // 0 poue simple (volets sélectionnés), 1 pour multiple (tous les volets)
    byte VoletDepart;                                       // volet de départ au niveau de la télécommande
    boolean Sel;                                            // sélection des volets
    byte i;

    ModeSelect = EEPROM.read(0);                            // extraction de la position du mode sur la télécommande mis en mémoire dans l'Eeprom
    VoletDepart = EEPROM.read(1);                           // extraction du volet de départ sur la télécommande mis en mémoire dans l'Eeprom
        if (ModeSelect == 1) {
            appuiTouche(PIN_VOL_MILIEU);                    // Appuie court sur bouton du milieu (Sélection Mode Simple)
        }
    EEPROM.write(0,0);                                      // mise en mémoire dans l'Eeprom du mode Simple
        if (VoletDepart != 1) {
            for (i = 0; i <= (VoletDepart - 2); i++){
                appuiTouche(PIN_VOL_GAUCHE);                // Appuie sur bouton de gauche pour amener sur la sélection du premier volet
            }
        }
    Prog = Prog -1;                                         // décalage de 1 pour correspondre avec l'emplacement de la mémoire EEPROM
        for (i = 0; i<8; i++){
            Sel = EEPROM.read(1004 + (Prog*24) + i);
            if (Sel == true) {
                if (Action == 'O') {
                    appuiTouche(PIN_VOL_HAUT);              // Appuie sur bouton Cde Haut (Ouvrir)
                }
                else if (Action == 'F') {
                    appuiTouche(PIN_VOL_BAS);               // Appuie sur bouton Cde Bas (Fermer)
                }
            }
            appuiTouche(PIN_VOL_DROIT);                     // Appuie sur bouton Cde droit pour changer de volet
        }
    EEPROM.write(1,1);                                      // mise en mémoire dans l'Eeprom dela position de départ (volet n°1)
}
// ----------------- Fin de la fonction action_prog_volet (ouverture ou fermeture programmée des volets) ------------------------------------------

// -------------------- Début de la fonction envoiProgVolet (envoi des programmations volets au serveur) --------------------------------------------
void envoiProgVolet()
{
    byte i, j;
    int Temp;

    // Elaboration du message sous la frme (ME01;PROGV;01;0;32/32/2032;00110100;12;40;15;10)
        for (i=0; i<8; i++) {
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "PROGV",5);
            EvtServer[11] = Separator;
			snprintf(&EvtServer[12],3,"%02d",i + 1);            // Numéro du programme
            EvtServer[14] = Separator;
            Temp = EEPROM.read(1000 + (i * 24));
			snprintf(&EvtServer[15],3,"%1d",Temp);              // activation 0 ou 1
            EvtServer[16] = Separator;
            Temp = EEPROM.read(1001 + (i * 24));
			snprintf(&EvtServer[17],3,"%02d",Temp);             // jour de la programmation (jour ou 32 pour semaine, 33 pour week-end
            EvtServer[19] = '/';
            Temp = EEPROM.read(1002 + (i * 24));
			snprintf(&EvtServer[20],3,"%02d",Temp);             // mois de la programmation (mois ou 32 pour semaine, 33 pour week-end
            EvtServer[22] = '/';
            Temp = (EEPROM.read(1003 + (i * 24))) + 2000;       // passage de l'année sur 4 chiffres
			snprintf(&EvtServer[23],5,"%4d",Temp);              // annee de la programmation (annee ou 32 pour semaine, 33 pour week-end
            EvtServer[27] = Separator;
                for (j=0; j<8; j++) {
                    Temp = EEPROM.read(1004 + (i * 24) + j);
                    snprintf(&EvtServer[28 + j],2,"%1d",Temp);  // sélection des volets 0 ou 1
                }
            EvtServer[36] = Separator;
            Temp = EEPROM.read(1020 + (i * 24));
			snprintf(&EvtServer[37],3,"%02d",Temp);             // heure ouverture (99 si pas de sélection)
            EvtServer[39] = Separator;
            Temp = EEPROM.read(1021 + (i * 24));
			snprintf(&EvtServer[40],3,"%02d",Temp);             // minute ouverture (99 si pas de sélection)
            EvtServer[42] = Separator;
            Temp = EEPROM.read(1022 + (i * 24));
			snprintf(&EvtServer[43],3,"%02d",Temp);             // heure fermeture (99 si pas de sélection)
            EvtServer[45] = Separator;
            Temp = EEPROM.read(1023 + (i * 24));
			snprintf(&EvtServer[46],3,"%02d",Temp);             // minute fermeture (99 si pas de sélection)
            EvtServer[48] = ETX;
            EvtServer[49] = '\0';
                #ifdef DEBUG_TELECOM
                    Serial.print(F("chaine emise = "));
                    Serial.println(EvtServer);
                #endif
                if (wsServer.isConnected()) {
                    wsServer.send(EvtServer,49);//Envoi
                }
        }
}
// -------------------- Fin de la fonction envoiProgVolet (envoi des programmations volets au serveur) ----------------------------------------------

// -------------------- Début de la fonction enregProgrammeVolet ( enregistrement de la programmation des volets en EEPROM --------------------------
void enregProgrammeVolet(char* Chaine)
{
    // réception d'une chaine sous la forme (ME01;ENPRV;01;1;12/10/2017;00110001;05;30;14;15)
    byte Chiffre;
    char Buff[3];
    byte NumProg;
    int Adresse;

    Buff[0] = Chaine[12];
    Buff[1] = Chaine[13];
    Buff[2] = '\0';
    NumProg = atoi(Buff);
    Adresse = 1000 + ((NumProg - 1) * 24);

    Buff[0] = '0';
    Buff[1] = Chaine[15];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);
    EEPROM.write(Adresse, Chiffre);
    Buff[0] = Chaine[17];
    Buff[1] = Chaine[18];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction du jour de programmation
    EEPROM.write(Adresse + 1, Chiffre);
    Buff[0] = Chaine[20];
    Buff[1] = Chaine[21];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction du mois de programmation
    EEPROM.write(Adresse + 2, Chiffre);
    Buff[0] = Chaine[25];
    Buff[1] = Chaine[26];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction de l'année de programmation (sur 2 chiffres)
    EEPROM.write(Adresse + 3, Chiffre);
    Buff[0] = '0';
        for (int i=0; i <8; i++){
            Buff[1] = Chaine[28 + i];
            Buff[2] = '\0';
            Chiffre= atoi(Buff);                            // Extraction de la sélection des volets (0 ou 1)
            EEPROM.write(Adresse + 4 + i, Chiffre);
        }
    Buff[0] = Chaine[37];
    Buff[1] = Chaine[38];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction de l'heure d'ouverture des volets
    EEPROM.write(Adresse + 20, Chiffre);
    Buff[0] = Chaine[40];
    Buff[1] = Chaine[41];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction des minutes d'ouverture des volets
    EEPROM.write(Adresse + 21, Chiffre);
    Buff[0] = Chaine[43];
    Buff[1] = Chaine[44];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction de l'heure de fermeture des volets
    EEPROM.write(Adresse + 22, Chiffre);
    Buff[0] = Chaine[46];
    Buff[1] = Chaine[47];
    Buff[2] = '\0';
    Chiffre= atoi(Buff);                                    // Extraction des minutes de fermeture des volets
    EEPROM.write(Adresse + 23, Chiffre);

        if (NumProg == 8) {
            creMessage("01", "envoiOK", 7);
            charger_prog_volet();                           // chargement de la programmation des volets pour la journée
        }
}
// -------------------- Fin de la fonction enregProgrammeVolet ( enregistrement de la programmation des volets en EEPROM ----------------------------

// ------------- Début de la fonction appuiTouche (appuie sur les touches des télécommandes volets ou portail ou portes de garage) ------------------
void appuiTouche(byte Bouton)
{
    digitalWrite(Bouton,HIGH);              // Appuie sur bouton
    delay(TempsApp);                        // durée de l'impulsion
    digitalWrite(Bouton,LOW);               // fin de l'appui
    delay(TempsApp);
}
// ------------- Fin de la fonction appuiTouche (appuie sur les touches des télécommandes volets ou portail ou portes de garage) --------------------
// *************************** Fin des fonctions pour la commande des volets ************************************************************************


// *************************** Début des fonctions pour la commande des portails ********************************************************************
// --------------------------- Début de la fonction commandePortail ---------------------------------------------------------------------------------
void commandePortail()
{
    appuiTouche(PIN_PORTAIL);                   // Appuie sur bouton de la télécommande du portail
    creMessage("01", "Commande portail", 16);
}
// --------------------------- Fin de la fonction commandePortail -----------------------------------------------------------------------------------
// *************************** Fin des fonctions pour la commande des portails **********************************************************************


// *************************** Début des fonctions pour la commande des portes de garage ************************************************************
// --------------------------- Début de la fonction commandeGarage ----------------------------------------------------------------------------------
void commandeGarage(byte NumGarage)
{
    switch (NumGarage)
    {
        case 1:
            appuiTouche(PIN_GARAGE_1);                                       // Appuie sur bouton de la télécommande des portes de garage
            creMessage("01", "Commande porte garage 1", 23);
            break;
        case 2:
            appuiTouche(PIN_GARAGE_2);                                       // Appuie sur bouton de la télécommande des portes de garage
            creMessage("01", "Commande porte garage 2", 23);
            break;
    }
}
// --------------------------- Fin de la fonction commandeGarage ------------------------------------------------------------------------------------
// *************************** Fin des fonctions pour la commande des portes de garage **************************************************************


// *************************** Début des fonctions pour la télecommande des prises électrique *******************************************************
// --------------------------- Début de la fonction commande_prises (commande à distance des prises électriques -------------------------------------
void commande_prises(char* Chaine)
{
    byte NbPrise;
    char Buff[4];
    char Cde[] = "ON";
    byte Ajout = 0;


    Buff[0] = '0';
    Buff[1] = Chaine[12];
    Buff[2] = Chaine[13];
    Buff[3] = '\0';
    NbPrise = atoi(Buff);

        if (strncmp(&Chaine[15],"ON",2) == 0){
            strcpy(Cde, "ON");
        }
        else {
            strcpy(Cde, "OFF");
        }

    // Elaboration du message
    strncpy(&EvtServer[0], "10;Eurodomest;",14);
    strncpy(&EvtServer[14], ID_INTER[NbPrise], 6);
    EvtServer[20] = Separator;
        if (Buff[1] != '0') {
            EvtServer[21] = Buff[1];
            Ajout = 1;
        }
    EvtServer[21 + Ajout] = Buff[2];
    EvtServer[22 + Ajout] = Separator;
        if (strncmp(&Chaine[15],"ON",2) == 0){
            strncpy(&EvtServer[23 + Ajout], "ON",2);
        }
        else {
            strncpy(&EvtServer[23 + Ajout], "OFF",3);
            Ajout = Ajout + 1;
        }
    EvtServer[25 + Ajout] = Separator;
    EvtServer[26 + Ajout] = '\0';

            #ifdef DEBUG_TELECOM
                Serial.print(F("Code : "));
                Serial.println(EvtServer);
            #endif

    Serial2.println(EvtServer);
}
// --------------------------- Fin de la fonction commande_prises (commande à distance des prises électriques ---------------------------------------
// *************************** Fin des fonctions pour la télecommande des prises électrique *********************************************************

