/*
*****************************************************************************************************************************************************
Projet Centrale domotique DomoMaison 2017 pour carte Arduino Mega 2560 et carte Ethernet (W5100)
Version 1.70 du 06/09/2017
Copyright 2017 PRIN Jean-Luc
Ce programme fonctionne avec le protocole Websockets (HTML5).
*****************************************************************************************************************************************************
*/

/*
*****************************************************************************************************************************************************
Modules contenant les fonctions nécessaires à la récupération et à la gestion des températures
*****************************************************************************************************************************************************
*/

// ------------------ Début de la fonction traitSondesTemperature (traitement de la réception des sondes de température) ----------------------------
void traiteSondesTemperature(char* Chaine)
{
	boolean Trouve = false;
    char TempC[] = "xxxx";
    byte TempId;
    int TempoT;
    byte TempHum;
    byte TempBatt;

    int i, j;

        #ifdef DEBUG_TEMP
            Serial.print(F("chaine Temp : "));
            Serial.write(Chaine);
            Serial.println();
        #endif // DEBUG_TEMP

        if (strncmp(&Chaine[6],"Oregon Temp;",12) == 0) {           // Réception Sonde températrure Oregon sans hygrométrie
            // chaine de type : "20;74;Oregon Temp;ID=0ACC;TEMP=00be;HUM=40;BAT=OK;"
            strncpy(TempC, &Chaine[21], 4);                         // Extraction de l'ID
            TempId = identi(TempC);
            strncpy(TempC, &Chaine[31], 4);                         // Extraction de la température
            TempC[4] = '\0';
            TempoT = temperature(TempC);
            TempHum = 0;
            strncpy(TempC, &Chaine[40], 2);                         // Extraction de la tension batterie
            TempC[2] = '\0';
            TempBatt = battery(TempC);
        }
        else if (strncmp(&Chaine[6],"Oregon TempHygro",16) == 0) {
            // chaine de type : "20;74;Oregon TempHygro;ID=0ACC;TEMP=00be;HUM=40;HSTATUS=0;BAT=OK;"
            strncpy(TempC, &Chaine[26], 4);                         // Extraction de ID
            TempId = identi(TempC);
            strncpy(TempC, &Chaine[36], 4);                         // Extraction de la température
            TempoT = temperature(TempC);
            strncpy(TempC, &Chaine[45], 2);                         // Extraction de l'humidité
            TempC[2] = '\0';
            TempHum = humidity(TempC);
            strncpy(TempC, &Chaine[62], 2);                         // Extraction de la tension batterie
            TempC[2] = '\0';
            TempBatt = battery(TempC);
        }

	// Mise en mémoires des sondes
        for (i=0; i < NB_SONDEST_S; i++){
			if(TempId == IdT[i + 5]) {                              // le numéro des sondes sans fil commence à 6 (1 à 5 = sondes filaires)
				TempT[i + 5] = TempoT;
				HumT[i + 5] = TempHum;
				BattT[i + 5] = TempBatt;
				Trouve = true;
                break;
			}
		}
		if(Trouve == false) {				                        // la sonde n'est encore en mémoire
			for (i=0; i < NB_SONDEST_S; i++){
				if(IdT[i + 5] == 0) {
					IdT[i + 5] = TempId;
					TempT[i + 5] = TempoT;
					HumT[i + 5] = TempHum;
					BattT[i + 5] = TempBatt;
					if (i == (NB_SONDEST_S - 1) && EnregSonde == 0) {   // Toutes les sondes ont été reçues, donc enregistrement en Eprom
                            // Enregistrement des données en Eprom
                            for (j=0; j < NB_SONDEST_S; j++) {
                                EEPROM.write(35 + j, IdT[j + 5]);
                            }
                            for (j=35 + NB_SONDEST_S; j <45; j++) {
                                EEPROM.write(j,0);
                            }
                        EEPROM.write(45, 1);                            // valide l'enregistrement nécessaire lors d'un reset de la centrale
                        EnregSonde = 1;
					}
                    break;
				}
			}
		}
}
// ------------------ Fin de la fonction traitSondesTemperature (traitement de la réception des sondes de température) ------------------------------

// ------------------- Début de la fonction identi (décodage de l'ID de la sonde de température sans fil) -------------------------------------------
byte identi(char* Data)
{
    int TpId = hexToDec(Data);
    byte Temp = byte(TpId);
	return Temp;
}
// ------------------- Fin de la fonction identi (décodage de l'ID de la sonde de température sans fil) ---------------------------------------------

// ------------------- Début de la fonction temperature (décodage de la température de la sonde sans fil) -------------------------------------------
int temperature(char* Data)
{
    int Temp;
    char Buff[] = "xxx";

    strncpy(Buff, &Data[1], 3);             // Extraction de la température
    Buff[3] = '\0';

    Temp = hexToDec(Buff);

		if(Data[0] == '8')
		{
			Temp = hexToDec(Buff) * -1;
		}

	return Temp;
}
// ------------------- Fin de la fonction temperature (décodage de la température de la sonde sans fil) ---------------------------------------------

// ------------------- Début de la fonction humidity (décodage de l'humidité de la sonde de température sans fil) -----------------------------------
byte humidity(char* Data)
{
	byte Temp = atoi(&Data[0]);

    return Temp;
}
// ------------------- Fin de la fonction humidity (décodage de l'humidité de la sonde de température sans fil) -----------------------------------

// ------------------- Début de la fonction battery (décodage du niveau de la batterie de la sonde de température sans fil) -------------------------
// Ne retourne qu'un apercu de l'etat de la baterie : 10 = faible
byte battery(char* Data)
{
	byte Temp;

		if (strncmp(&Data[0],"LO",2) == 0)
		{
			Temp = 10;          // batterie faible
		}
		else
		{
			Temp = 90;          // batterie OK
		}

    return Temp;
}
// ------------------- Fin de la fonction battery (décodage du niveau de la batterie de la sonde de température sans fil) ---------------------------

// ------------------ Début de la fonction hexToDec (conversion d'une chaine hexadécimal en décimal) ------------------------------------------------
unsigned int hexToDec(char* HexString)
{
    unsigned int DecValue = 0;
    int NextInt;
    byte LgCh;

    LgCh = strlen(HexString);

        for (int i = 0; i < LgCh; i++) {
            NextInt = int(HexString[i]);
                if (NextInt >= 48 && NextInt <= 57) NextInt = map(NextInt, 48, 57, 0, 9);
                if (NextInt >= 65 && NextInt <= 70) NextInt = map(NextInt, 65, 70, 10, 15);
                if (NextInt >= 97 && NextInt <= 102) NextInt = map(NextInt, 97, 102, 10, 15);
            NextInt = constrain(NextInt, 0, 15);
            DecValue = (DecValue * 16) + NextInt;
        }

  return DecValue;
}
// ------------------ Fin de la fonction hexToDec (conversion d'une chaine hexadécimal en décimal) --------------------------------------------------

// --------------Début de la fonction envoi_appairage_sondesT (envoi de l'appairage des sondes de T° sans filau serveur -----------------------------
void envoi_appairage_sondesT(byte NbrePieces)
{
    int Temp;
    char Rang_Char[4];
    byte NumS;

        for (int i=0; i < NbrePieces; i++){
            // Elaboration du message d'appairage de la sonde de température
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "PARAM",5);
            EvtServer[11] = Separator;
            strncpy(&EvtServer[12], "02",2);        // 02 pour signaler un envoi de paramètres appairage de sondes
            EvtServer[14] = Separator;
            // Numéro de la pièce
			snprintf(&EvtServer[15],3,"%2d",i + 1); // le numéro de la pièce commence à 1
            EvtServer[17] = Separator;
            // Numéro sonde
            NumS = EEPROM.read(20 + i);
			snprintf(&EvtServer[18],3,"%2d",NumS);
            EvtServer[20] = Separator;
            // ID de la sonde
			snprintf(&EvtServer[21],4,"%3d",IdT[NumS - 1]);
            EvtServer[24] = ETX;
            EvtServer[25] = '\0';
                #ifdef DEBUG_TEMP
                    Serial.print(F("chaine emise = "));
                    Serial.println(EvtServer);
                #endif
                if (wsServer.isConnected()) {
                    wsServer.send(EvtServer,25);                //Envoi
                }
        }
}
// ------------- Fin de la fonction envoi_appairage_sondesT (envoi de l'appairage des sondes de T° sans fil au serveur ------------------------------

// ------------------ Début de la fonction envoi_web_sondesT (envoi vers le serveur web des info. des sondes de température) ------------------------
void envoi_web_sondesT()
{
        for (int i=0; i < 15; i++){
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "SONTE",5);
            EvtServer[11] = Separator;
			snprintf(&EvtServer[12],3,"%2d",i + 1);         // numéro de la sonde
            EvtServer[14] = Separator;
			snprintf(&EvtServer[15],4,"%3d",IdT[i]);        // id de la sonde
            EvtServer[18] = Separator;
			snprintf(&EvtServer[19],5,"%4d",TempT[i]);      // température de la sonde
            EvtServer[23] = Separator;
			snprintf(&EvtServer[24],3,"%2d",HumT[i]);       // humidité de la sonde
            EvtServer[26] = Separator;
			snprintf(&EvtServer[27],3,"%2d",BattT[i]);      // niveau batterie de la sonde
			EvtServer[29] = ETX;
			EvtServer[30] = '\0';
				if (wsServer.isConnected()) {
					wsServer.send(EvtServer,30);            //Envoi
				}
            #ifdef DEBUG_TEMP
                Serial.print(F("chaine sondes T = "));
                Serial.println(EvtServer);
            #endif
		}
}
// ------------------ Fin de la fonction envoi_web_sondesT (envoi vers le serveur web des info. des sondes de température) --------------------------

// ----------------- Début de la fonction initSondesT (réinitialisation des sondes de température) --------------------------------------------------
void initSondesT()
{
    byte i;

    // Effacement des données en Eprom
    for (i=20; i < 46; i++){
        EEPROM.write(i,0);
    }
    // Effacement du Tableau des sondes
    for (i=0; i < 15; i++){
        IdT[i] = 0;	                        // identifiant unique de la sonde
        TempT[i] = 0;	                    // température émise par la sonde
        HumT[i] = 0;						// humidité émise par la sonde
        BattT[i] = 10;						// niveau de la batterie de la sonde (10=faible, 90=OK)
    }
}
// ----------------- Fin de la fonction initSondesT (réinitialisation des sondes de température) ----------------------------------------------------

// --------------- Début de la fonction sauveParametresSondesT (sauvegarde des paramètres d'initialisation des sondes de T°) ------------------------
void sauve_appairage_sondesT(char* Chaine)
{
    char Buff[] = "xx";
    byte Adresse;
    byte Num;

    // Chaine reçue sous forme : (ME01;EPARA;02; 1; 7; 8; 10; 9;13; 0; 0; 0; 0; 0; 0; 0; 0; 0)
        for (byte i=0; i < 15; i++){
            Buff[0] = Chaine[15 + (i*3)];
            Buff[1] = Chaine[16 + (i*3)];
            Buff[2] = '\0';
            Num = atoi(Buff);
            Adresse = 20 + i;
            EEPROM.write(Adresse, Num);
                #ifdef DEBUG_TEMP
                    Serial.print(F("adresse : "));
                    Serial.print(Adresse);
                    Serial.print(F(" ;  Num : "));
                    Serial.print(Num);
                #endif
        }

    creMessage("02", "envoiOK", 7);
}
// --------------- Fin de la fonction sauveParametresSondesT (sauvegarde des paramètres d'initialisation des sondes de T°) ------------------------
